/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dime.h:
	External declarations for the Dime library.

	By Sven Sandberg
*/

#ifndef _DIME_H_
#define _DIME_H_

/* For `INT_MIN'. */
#include <limits.h>
/* For `struct FONT'. */
#include <allegro.h>

#ifdef __cplusplus
	extern "C" {
#endif

#define DIME_VERSION          1
#define DIME_SUB_VERSION      2
#define DIME_SUB_SUB_VERSION  1
/* #define DIME_BETA_VERSION */
#define DIME_VERSION_STRING   "1.2.1"
#define DIME_DATE             20000611
#define _DIME_STRINGIFY(x)    #x
#define DIME_DATE_STRING      _DIME_STRINGIFY(DIME_DATE)
#undef _DIME_STRINGIFY



#include "dime/dimecfg.h"



/******************************************/
/************ Dialog Functions ************/
/******************************************/

/* Setup and setdown. */
DIME_FUNC(int, dime_init, (void));
DIME_FUNC(void, dime_exit, (void));

/* Input dialogs. */
DIME_FUNC(int, dialogf, (char *title, int x, int y, int edit_w, char *format, ...));
DIME_FUNC(int, vdialogf, (char *title, int x, int y, int edit_w, char *format, va_list args));

/* Message boxes. */
#if 0 /* This is not finished yet. */
#define MESSAGE_BOX_MAX_TEXT_LENGTH (8192)
DIME_FUNC(int, message_box, (char *title, char *buttons, char *format, ...));
DIME_FUNC(int, vmessage_box, (char *title, char *buttons, char *format, va_list args));
#endif /* if 0 */

/* Variations of Allegro's dialog functions. */
DIME_FUNC(int, do_dialog_find_focus_object, (DIALOG *dialog, int focus_obj));
DIME_FUNC(DIALOG_PLAYER *, init_dialog_find_focus_object, (DIALOG *dialog, int focus_obj));

/* Dialog appearance. */
DIME_VAR(RGB, dime_text_rgb);
DIME_VAR(RGB, dime_disabled_text_rgb);
DIME_VAR(RGB, dime_error_text_rgb);

DIME_VAR(RGB, dime_border_rgb);
DIME_VAR(RGB, dime_background_rgb);
DIME_VAR(RGB, dime_edit_field_rgb);
DIME_VAR(RGB, dime_button_rgb);
DIME_VAR(RGB, dime_dark_shadow_rgb);
DIME_VAR(RGB, dime_shadow_rgb);
DIME_VAR(RGB, dime_highlight_rgb);
DIME_VAR(RGB, dime_light_highlight_rgb);
DIME_VAR(BITMAP *, dime_bmp);
DIME_VAR(FONT *, dime_font);
DIME_VAR(FONT *, dime_title_font);
DIME_VAR(FONT *, dime_button_font);
#define DIME_BMP  (dime_bmp == NULL ? screen : dime_bmp)
#define DIME_FONT (dime_font == NULL ? font : dime_font)

DIME_VAR(int, dime_yield);
DIME_VAR(int, dime_clean_screen);
DIME_FUNCPTR(void, dialogf_callback, (DIALOG *d));

DIME_VAR(int, dime_window_border_thickness);
DIME_VAR(int, dime_window_title_spacing);
DIME_VAR(int, dime_window_button_spacing);
DIME_VAR(int, dime_window_between_button_spacing);
DIME_VAR(int, dime_window_description_spacing);
DIME_VAR(int, dime_window_line_spacing);
DIME_VAR(int, dime_window_button_w);
DIME_VAR(int, dime_window_button_h);



/****************************************/
/************ Misc Functions ************/
/****************************************/

/* Helper functions for converting between Allegro scancodes and strings. */
DIME_FUNC(char *, scancode_to_short_name, (int scancode, char *buf));
DIME_FUNC(int, short_name_to_scancode, (char *short_name));
DIME_FUNC(char *, scancode_to_pretty_name, (int scancode, char *buf));

/* Helper functions for evaluating expressions. */
DIME_FUNC(double, evaluate, (char *equation, int *error, DIME_METHOD(double, variable, (char *name))));
/* Helper function for drawing objects. */
DIME_FUNC(void, fill_textout, (BITMAP *bmp, FONT *f, char *text, int x, int y, int w, int fg, int bg));
/* Helper function for getting the dimensions of serveral lines of text. */
DIME_FUNC(void, multiline_text_size, (FONT *f, char *text, int *w, int *h));

/* Helper functions for printing floating point numbers. */
DIME_FUNC(void, double2string, (double x, char *buf));
DIME_FUNC(void, udouble2string, (double val, char *buf));

/* Helper functions returning the pixel width of single characters and of
	substrings. */
DIME_FUNC(int, dime_char_width, (FONT *f, int c));
DIME_FUNC(int, dime_substr_width, (FONT *f, char *text, int start, int length));
/* Helper function to draw only part of a string. */
DIME_FUNC(void, dime_draw_substr, (BITMAP *bmp, FONT *f, char *text, int x, int y, int color, int start, int length));

/* Alignment constants. */
#define ALIGN_LEFT                (INT_MIN)
#define ALIGN_LEFT_CENTER         (INT_MIN + 1)
#define ALIGN_LEFT_RIGHT          (INT_MIN + 2)
#define ALIGN_CENTER              (INT_MIN + 3)
#define ALIGN_CENTER_LEFT         (INT_MIN + 4)
#define ALIGN_CENTER_RIGHT        (INT_MIN + 5)
#define ALIGN_RIGHT_LEFT          (INT_MIN + 6)
#define ALIGN_RIGHT_CENTER        (INT_MIN + 7)
#define ALIGN_RIGHT               (INT_MIN + 8)
#define ALIGN_WORDJUSTIFY         (INT_MIN + 9)
#define ALIGN_WORDJUSTIFY_LEFT    (INT_MIN + 10)
#define ALIGN_WORDJUSTIFY_CENTER  (INT_MIN + 11)
#define ALIGN_WORDJUSTIFY_RIGHT   (INT_MIN + 12)
#define ALIGN_CHARJUSTIFY         (INT_MIN + 13)
#define ALIGN_CHARJUSTIFY_LEFT    (INT_MIN + 14)
#define ALIGN_CHARJUSTIFY_CENTER  (INT_MIN + 15)
#define ALIGN_CHARJUSTIFY_RIGHT   (INT_MIN + 16)
#define ALIGN_TOP                 (INT_MIN + 17)
#define ALIGN_BOTTOM              (INT_MIN + 18)
#define ALIGN_JUSTIFY             (ALIGN_WORDJUSTIFY)

#define ALIGN_CENTRE              (ALIGN_CENTER)
#define ALIGN_CENTRE_LEFT         (ALIGN_CENTER_LEFT)
#define ALIGN_CENTRE_RIGHT        (ALIGN_CENTER_RIGHT)
#define ALIGN_LEFT_CENTRE         (ALIGN_LEFT_CENTER)
#define ALIGN_RIGHT_CENTRE        (ALIGN_RIGHT_CENTER)
#define ALIGN_WORDJUSTIFY_CENTRE  (ALIGN_WORDJUSTIFY_CENTER)
#define ALIGN_CHARJUSTIFY_CENTRE  (ALIGN_CHARJUSTIFY_CENTER)



/***************************************/
/************ Gui Functions ************/
/***************************************/

/* Button drawing. */
DIME_FUNC(void, draw_empty_button_frame, (BITMAP *bmp, int x1, int y1, int x2, int y2, int face, int xlight, int light, int dark, int xdark));
DIME_FUNC(void, draw_empty_button, (BITMAP *bmp, int x1, int y1, int x2, int y2, int face, int xlight, int light, int dark, int xdark));
DIME_FUNC(void, draw_text_button, (BITMAP *bmp, int x1, int y1, int x2, int y2, int face, int textcolor, int xlight, int light, int dark, int xdark, FONT *f, char *text));
DIME_FUNC(void, draw_text_button_down, (BITMAP *bmp, int x1, int y1, int x2, int y2, int face, int textcolor, int xlight, int light, int dark, int xdark, FONT *f, char *text));
DIME_FUNC(void, draw_picture_button, (BITMAP *bmp, int x1, int y1, int x2, int y2, int face,int xlight, int light, int dark, int xdark, BITMAP *sprite));
DIME_FUNC(void, draw_picture_button_down, (BITMAP *bmp, int x1, int y1, int x2, int y2, int face, int xlight, int light, int dark, int xdark, BITMAP *sprite));
/* Gui procs. */
DIME_FUNC(int, d_line_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_multiline_text_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_dime_button_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_dime_edit_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_dime_text_list_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_dime_check_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_double_calc_edit_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_int_calc_edit_proc, (int msg, DIALOG *d, int c));
DIME_FUNC(int, d_calc_edit_result_proc, (int msg, DIALOG *d, int c));

typedef struct DIALOGF_EDIT_NUMBER
{
	int is_signed;          /* Set for integer formats if it is signed. */
	int is_float;           /* Set if we only have float, not double. */
	double min_val, max_val;/* Range for numbers. */
	DIALOG *result_object;  /* DIALOG object to display result in. */
} DIALOGF_EDIT_NUMBER;



/*******************************************/
/************ dialogf() Formats ************/
/*******************************************/

/* Internal structure for each format type. */
typedef struct _DIALOGF_FORMAT
{
	/* The format string to identify us with. */
	char *specifier;
	/* If set, the object prefers to take care of placing the description
		itself, rather than letting the dialog handler do it. Note that if this
		is set, then `dialog_count()' must also count the number of objects
		needed for the description. */
	int handle_desc;
	/* Returns the number of dialog objects to allocate. */
	DIME_METHOD(int, count_dialog, (char *text));
	/* Creates the DIALOG objects. */
	DIME_METHOD(int, create_dialog, (DIALOG *dialog, char *desc, char *arg_text, dime_va_list_p args));
	/* Possibly set to a function operating after the dialog has temporarily
		been shut down. */
	DIME_METHOD(int, outside_dialog, (DIALOG *object));
	/* Stores the results in the parameters. */
	DIME_METHOD(void, store_dialog_result, (DIALOG *object));
	/* Resets the dialog to its default values. */
	DIME_METHOD(void, reset_dialog, (DIALOG *object));
	/* Frees memory. */
	DIME_METHOD(void, destroy_dialog_data, (DIALOG *object));

	struct _DIALOGF_FORMAT *next;
} _DIALOGF_FORMAT;

DIME_ARRAY(_DIALOGF_FORMAT, _dialogf_format_list);

/* A list of all formats. This is internal only! */
DIME_VAR(_DIALOGF_FORMAT *, _dialogf_formats);


/* Macros for listing all formats. */
#define BEGIN_DIALOGF_FORMAT_LIST \
	_DIALOGF_FORMAT _dialogf_format_list[] = \
	{

#define END_DIALOGF_FORMAT_LIST \
		{  NULL, FALSE, NULL, NULL, \
			NULL, NULL, NULL, NULL, NULL } \
	};

#define DIALOGF_FORMAT_BOOL \
	{  "bool", FALSE, dialogf_return_1, dialogf_create_bool, \
		NULL, dialogf_store_bool, dialogf_reset_bool, NULL, NULL },
#define DIALOGF_FORMAT_STRING \
	{  "string", FALSE, dialogf_return_1, dialogf_create_string, \
		NULL, dialogf_store_string, dialogf_reset_string, dialogf_destroy_dp, \
		NULL },
#define DIALOGF_FORMAT_FILENAME \
	{  "filename", FALSE, dialogf_return_2, dialogf_create_filename, \
		dialogf_outside_filename, dialogf_store_filename, \
		dialogf_reset_string, dialogf_destroy_filename, NULL },
#define DIALOGF_FORMAT_VLIST \
	{  "vlist", FALSE, dialogf_return_1, dialogf_create_vlist, \
		NULL, dialogf_store_lists, dialogf_reset_lists, \
		dialogf_destroy_vlist, NULL },
#define DIALOGF_FORMAT_LIST \
	{  "list", FALSE, dialogf_return_1, dialogf_create_list, \
		NULL, dialogf_store_lists, dialogf_reset_lists, \
		dialogf_destroy_list_and_datafile, NULL },
#define DIALOGF_FORMAT_DATAFILE \
	{  "datafile", FALSE, dialogf_return_1, dialogf_create_datafile, \
		NULL, dialogf_store_datafiles, dialogf_reset_datafile, \
		dialogf_destroy_list_and_datafile, NULL },
#define DIALOGF_FORMAT_WVLIST \
	{  "wvlist", FALSE, dialogf_return_1, dialogf_create_wvlist, \
		dialogf_outside_wlists, dialogf_store_lists, dialogf_reset_wlists, \
		dialogf_destroy_wvlist, NULL },
#define DIALOGF_FORMAT_WLIST \
	{  "wlist", FALSE, dialogf_return_1, dialogf_create_wlist, \
		dialogf_outside_wlists, dialogf_store_lists, dialogf_reset_wlists, \
		dialogf_destroy_wlist_and_wdatafile, NULL },
#define DIALOGF_FORMAT_WDATAFILE \
	{  "wdatafile", FALSE, dialogf_return_1, dialogf_create_wdatafile, \
		dialogf_outside_wlists, dialogf_store_datafiles, \
		dialogf_reset_wdatafile, dialogf_destroy_wlist_and_wdatafile, NULL },
#define DIALOGF_FORMAT_DIALOGF \
	{  "dialogf", FALSE, dialogf_return_1, dialogf_create_dialogf, \
		dialogf_outside_dialogf, dialogf_store_dialogf, \
		dialogf_reset_dialogf, dialogf_destroy_dialogf, NULL },
#define DIALOGF_FORMAT_VDIALOGF \
	{  "vdialogf", FALSE, dialogf_return_1, dialogf_create_vdialogf, \
		dialogf_outside_dialogf, dialogf_store_dialogf, \
		dialogf_reset_dialogf, dialogf_destroy_dialogf, NULL },
#define DIALOGF_FORMAT_INT \
	{  "int", FALSE, dialogf_return_2, dialogf_create_int, NULL, \
		dialogf_store_int, dialogf_reset_int, dialogf_destroy_calc, NULL },
#define DIALOGF_FORMAT_PINT \
	{  "pint", FALSE, dialogf_return_1, dialogf_create_pint, NULL, \
		dialogf_store_pint, dialogf_reset_int, dialogf_destroy_plain, NULL },
#define DIALOGF_FORMAT_UINT \
	{  "uint", FALSE, dialogf_return_2, dialogf_create_uint, NULL, \
		dialogf_store_uint, dialogf_reset_uint, dialogf_destroy_calc, NULL },
#define DIALOGF_FORMAT_PUINT \
	{  "puint", FALSE, dialogf_return_1, dialogf_create_puint, NULL, \
		dialogf_store_puint, dialogf_reset_uint, dialogf_destroy_plain, NULL },
#define DIALOGF_FORMAT_SHORT \
	{  "short", FALSE, dialogf_return_2, dialogf_create_short, NULL, \
		dialogf_store_short, dialogf_reset_short, dialogf_destroy_calc, NULL },
#define DIALOGF_FORMAT_PSHORT \
	{  "pshort", FALSE, dialogf_return_1, dialogf_create_pshort, NULL, \
		dialogf_store_pshort, dialogf_reset_short, dialogf_destroy_plain, \
		NULL },
#define DIALOGF_FORMAT_USHORT \
	{  "ushort", FALSE, dialogf_return_2, dialogf_create_ushort, NULL, \
		dialogf_store_ushort, dialogf_reset_ushort, dialogf_destroy_calc, \
		NULL },
#define DIALOGF_FORMAT_PUSHORT \
	{  "pushort", FALSE, dialogf_return_1, dialogf_create_pushort, NULL, \
		dialogf_store_pushort, dialogf_reset_ushort, dialogf_destroy_plain, \
		NULL },
#define DIALOGF_FORMAT_CHAR \
	{  "char", FALSE, dialogf_return_2, dialogf_create_char, NULL, \
		dialogf_store_char, dialogf_reset_char, dialogf_destroy_calc, NULL },
#define DIALOGF_FORMAT_PCHAR \
	{  "pchar", FALSE, dialogf_return_1, dialogf_create_pchar, NULL, \
		dialogf_store_pchar, dialogf_reset_char, dialogf_destroy_plain, NULL },
#define DIALOGF_FORMAT_UCHAR \
	{  "uchar", FALSE, dialogf_return_2, dialogf_create_uchar, NULL, \
		dialogf_store_uchar, dialogf_reset_uchar, dialogf_destroy_calc, NULL },
#define DIALOGF_FORMAT_PUCHAR \
	{  "puchar", FALSE, dialogf_return_1, dialogf_create_puchar, \
		NULL, dialogf_store_puchar, dialogf_reset_uchar, \
		dialogf_destroy_plain, NULL },
#ifdef DIME_LONG_LONG
	#define DIALOGF_FORMAT_LONGLONG \
		{  "longlong", FALSE, dialogf_return_2, dialogf_create_longlong, \
			NULL, dialogf_store_longlong, dialogf_reset_longlong, \
			dialogf_destroy_calc, NULL },
	#define DIALOGF_FORMAT_PLONGLONG \
		{  "plonglong", FALSE, dialogf_return_1, dialogf_create_plonglong, \
			NULL, dialogf_store_plonglong, dialogf_reset_longlong, \
			dialogf_destroy_plain, NULL },
	#define DIALOGF_FORMAT_ULONGLONG \
		{  "ulonglong", FALSE, dialogf_return_2, dialogf_create_ulonglong, \
			NULL, dialogf_store_ulonglong, dialogf_reset_ulonglong, \
			dialogf_destroy_calc, NULL },
	#define DIALOGF_FORMAT_PULONGLONG \
		{  "pulonglong", FALSE, dialogf_return_1, dialogf_create_pulonglong, \
			NULL, dialogf_store_pulonglong, dialogf_reset_ulonglong, \
			dialogf_destroy_plain, NULL },
#endif /* ifdef DIME_LONG_LONG */
#define DIALOGF_FORMAT_DOUBLE \
	{  "double", FALSE, dialogf_return_2, dialogf_create_double, \
		NULL, dialogf_store_double, dialogf_reset_double, \
		dialogf_destroy_calc, NULL },
#define DIALOGF_FORMAT_PDOUBLE \
	{  "pdouble", FALSE, dialogf_return_1, dialogf_create_pdouble, \
		NULL, dialogf_store_double, dialogf_reset_double, \
		dialogf_destroy_plain, NULL },
#define DIALOGF_FORMAT_FLOAT \
	{  "float", FALSE, dialogf_return_2, dialogf_create_float, \
		NULL, dialogf_store_float, dialogf_reset_float, dialogf_destroy_calc, \
		NULL },
#define DIALOGF_FORMAT_PFLOAT \
	{  "pfloat", FALSE, dialogf_return_1, dialogf_create_pfloat, \
		NULL, dialogf_store_float, dialogf_reset_float, \
		dialogf_destroy_plain, NULL },
#define DIALOGF_FORMAT_LINE \
	{  "line", TRUE, dialogf_return_2, dialogf_create_line, \
		NULL, NULL, NULL, dialogf_destroy_dp, NULL },
#define DIALOGF_FORMAT_NOTHING \
	{  "nothing", TRUE, dialogf_return_1, dialogf_create_nothing, \
		NULL, NULL, NULL, dialogf_destroy_dp, NULL },

/* Helpers common to several formats. */
DIME_FUNC(int, dialogf_create_pinteger, (DIALOG *d, char *arg_text, dime_va_list_p args, double min_val, double max_val));
DIME_FUNC(int, dialogf_create_integer, (DIALOG *d, char *arg_text, dime_va_list_p args, double min_val, double max_val));

DIME_FUNC(int, dialogf_create_pfloatingpoint, (DIALOG *d, char *arg_text, dime_va_list_p args, int is_double));
DIME_FUNC(int, dialogf_create_floatingpoint, (DIALOG *d, char *arg_text, dime_va_list_p args, int is_double));

DIME_FUNC(void, dialogf_destroy_plain, (DIALOG *d));
DIME_FUNC(void, dialogf_destroy_calc, (DIALOG *d));

DIME_FUNC(int, dialogf_outside_wlists, (DIALOG *d));
DIME_FUNC(void, dialogf_store_lists, (DIALOG *d));
DIME_FUNC(void, dialogf_store_datafiles, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_lists, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_wlists, (DIALOG *d));
DIME_FUNC(void, dialogf_destroy_list_and_datafile, (DIALOG *d));
DIME_FUNC(void, dialogf_destroy_wlist_and_wdatafile, (DIALOG *d));

/* The formats. */
DIME_FUNC(int, dialogf_create_bool, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_bool, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_bool, (DIALOG *d));

DIME_FUNC(int, dialogf_create_string, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_string, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_string, (DIALOG *d));

DIME_FUNC(int, dialogf_create_filename, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(int, dialogf_outside_filename, (DIALOG *d));
DIME_FUNC(void, dialogf_store_filename, (DIALOG *d));
DIME_FUNC(void, dialogf_destroy_filename, (DIALOG *d));

DIME_FUNC(int, dialogf_create_vlist, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_destroy_vlist, (DIALOG *d));

DIME_FUNC(int, dialogf_create_list, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));

DIME_FUNC(int, dialogf_create_datafile, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_reset_datafile, (DIALOG *d));

DIME_FUNC(int, dialogf_create_wvlist, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_destroy_wvlist, (DIALOG *d));

DIME_FUNC(int, dialogf_create_wlist, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));

DIME_FUNC(int, dialogf_create_wdatafile, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_reset_wdatafile, (DIALOG *d));

DIME_FUNC(int, dialogf_create_dialogf, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(int, dialogf_create_vdialogf, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(int, dialogf_outside_dialogf, (DIALOG *d));
DIME_FUNC(void, dialogf_store_dialogf, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_dialogf, (DIALOG *d));
DIME_FUNC(void, dialogf_destroy_dialogf, (DIALOG *d));

DIME_FUNC(int, dialogf_create_int, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_int, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_int, (DIALOG *d));

DIME_FUNC(int, dialogf_create_pint, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_pint, (DIALOG *d));

DIME_FUNC(int, dialogf_create_uint, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_uint, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_uint, (DIALOG *d));

DIME_FUNC(int, dialogf_create_puint, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_puint, (DIALOG *d));

DIME_FUNC(int, dialogf_create_short, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_short, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_short, (DIALOG *d));

DIME_FUNC(int, dialogf_create_pshort, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_pshort, (DIALOG *d));

DIME_FUNC(int, dialogf_create_ushort, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_ushort, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_ushort, (DIALOG *d));

DIME_FUNC(int, dialogf_create_pushort, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_pushort, (DIALOG *d));

DIME_FUNC(int, dialogf_create_char, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_char, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_char, (DIALOG *d));

DIME_FUNC(int, dialogf_create_pchar, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_pchar, (DIALOG *d));

DIME_FUNC(int, dialogf_create_uchar, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_uchar, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_uchar, (DIALOG *d));

DIME_FUNC(int, dialogf_create_puchar, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_puchar, (DIALOG *d));

/* long long is a bit tricky to implement and doesn't work yet. */
#ifdef DIME_LONG_LONG
	DIME_FUNC(int, dialogf_create_longlong, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
	DIME_FUNC(void, dialogf_store_longlong, (DIALOG *d));
	DIME_FUNC(void, dialogf_reset_longlong, (DIALOG *d));

	DIME_FUNC(int, dialogf_create_plonglong, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
	DIME_FUNC(void, dialogf_store_plonglong, (DIALOG *d));

	DIME_FUNC(int, dialogf_create_ulonglong, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
	DIME_FUNC(void, dialogf_store_ulonglong, (DIALOG *d));
	DIME_FUNC(void, dialogf_reset_ulonglong, (DIALOG *d));

	DIME_FUNC(int, dialogf_create_pulonglong, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
	DIME_FUNC(void, dialogf_store_pulonglong, (DIALOG *d));
#endif

DIME_FUNC(int, dialogf_create_double, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_double, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_double, (DIALOG *d));

DIME_FUNC(int, dialogf_create_pdouble, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_pdouble, (DIALOG *d));

DIME_FUNC(int, dialogf_create_float, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_float, (DIALOG *d));
DIME_FUNC(void, dialogf_reset_float, (DIALOG *d));

DIME_FUNC(int, dialogf_create_pfloat, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_pfloat, (DIALOG *d));

DIME_FUNC(int, dialogf_create_nothing, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));

DIME_FUNC(int, dialogf_create_line, (DIALOG *d, char *desc, char *arg_text, dime_va_list_p args));

/* Helpers that do very trivial things for the count and destroy functions.
*/
DIME_FUNC(int, dialogf_return_1, (char *text));
DIME_FUNC(int, dialogf_return_2, (char *text));
DIME_FUNC(void, dialogf_destroy_dp, (DIALOG *d));



/********************************************/
/************ Edit Box Functions ************/
/********************************************/

#if 0 /* This is not official yet. */

/* Clipboard. */
#define CLIPBOARD_TYPE_EMPTY     (0)
#define CLIPBOARD_TYPE_TEXT      (1)
#define CLIPBOARD_TYPE_NUMBER    (2)
#define CLIPBOARD_TYPE_BITMAP    (3)
#define CLIPBOARD_TYPE_SAMPLE    (4)
typedef DIME_METHOD(void, CLIPBOARD_ITEM_DESTROYER, (void *data));
typedef struct CLIPBOARD_ITEM          /* One thing in clipboard. */
{
	int type;                           /* One of the constants above. */
	void *data;                         /* Contents. */
	CLIPBOARD_ITEM_DESTROYER destroy;   /* Function for freeing the data.*/
	struct CLIPBOARD_ITEM *next;        /* There may be >1 thing in clipboard
														at a time. */
} CLIPBOARD_ITEM;

/* A complete clipboard system. */
typedef struct CLIPBOARD
{
	int max_history;        /* Clipboard may remember old contents. */
	int num_items;          /* Current number of remembered items. */
	CLIPBOARD_ITEM *item;   /* Array of items. */
} CLIPBOARD;

DIME_FUNC(int, clipboard_has_contents, (CLIPBOARD *c));
DIME_FUNC(int, clipboard_has_type, (CLIPBOARD *c, int type));
DIME_FUNC(void *, clipboard_get_data, (CLIPBOARD *c, int type));

DIME_FUNC(int, clipboard_put_text, (CLIPBOARD *c, char *text));
DIME_FUNC(int, clipboard_put_bitmap, (CLIPBOARD *c, BITMAP *bmp));
DIME_FUNC(int, clipboard_put_sample, (CLIPBOARD *c, SAMPLE *s));
DIME_FUNC(int, clipboard_put_item, (CLIPBOARD *c, void *data, int type, CLIPBOARD_ITEM_DESTROYER destroy));
DIME_FUNC(int, clipboard_put_subitem, (CLIPBOARD *c, void *data, int type, CLIPBOARD_ITEM_DESTROYER destroy));

DIME_FUNC(CLIPBOARD *, create_clipboard, (void));
DIME_FUNC(CLIPBOARD *, create_history_clipboard, (int max_history));
DIME_FUNC(void, destroy_clipboard, (CLIPBOARD *c));
DIME_FUNC(void, destroy_clipboard_item, (CLIPBOARD_ITEM *ci));

/* Edit box. */
#define EDIT_TYPE_TEXT              (1)
#define EDIT_TYPE_NUM               (2)
#define EDIT_TYPE_FILE              (3)

#define EDIT_INSERT                 (1)
#define EDIT_OVERWRITE              (2)
#define EDIT_ONLY_INSERT            (TRUE)   /* Can't switch to overwrite. */
#define EDIT_ONLY_OVERWRITE         (FALSE)  /* Can't switch to insert. */

#define EDIT_NUM_DECIMALS           (1) /* Real numbers. */
#define EDIT_NUM_NODECIMALS         (2) /* Integers. */
#define EDIT_NUM_HALVES             (3) /* Using '1/2'. */
#define EDIT_NUM_QUARTERS           (4) /* Using '1/4', '1/2' and '3/4'. */

#define EDIT_FILE_NAME              (0x0001)
#define EDIT_FILE_PATH              (0x0002)
#define EDIT_FILE_DRIVE             (0x0004)
#define EDIT_FILE_DOS               (1)
#define EDIT_FILE_UNIX              (2)
#define EDIT_FILE_WINDOWS           (3)
#define EDIT_FILE_CURRENT           (4)
#define EDIT_FILE_WILDCARDS         (TRUE)
#define EDIT_FILE_NOWILDCARDS       (FALSE)

#define EDIT_TEXT_SPACE             (0x0001)
#define EDIT_TEXT_SYMBOLS           (0x0002)
#define EDIT_TEXT_DIGITS            (0x0004)
#define EDIT_TEXT_UPPER             (0x0008)
#define EDIT_TEXT_LOWER             (0x0010)
#define EDIT_TEXT_ALPHA             (EDIT_TEXT_UPPER | EDIT_TEXT_LOWER)
#define EDIT_TEXT_ALL               (EDIT_TEXT_SPACE | EDIT_TEXT_SYMBOLS | \
												 EDIT_TEXT_DIGITS | EDIT_TEXT_ALPHA)

#define EDIT_CLIPBOARD              (3)   /* Full clipboard support. */
#define EDIT_SELECT                 (1)   /* No clipboard, but selections. */
#define EDIT_NOSELECT               (0)   /* Not even selections. */

struct EDITBOX;
struct EDITBOX_OPTIONS;

/* Function pointers to most things that can happen in an edit box. */
typedef struct EDITBOX_FUNCTIONS
{
	DIME_METHOD(int, edit, (struct EDITBOX *ed, int key, int shifts));
	DIME_METHOD(int, key_text, (struct EDITBOX *ed, int key, int shifts));
	DIME_METHOD(int, key_nontext, (struct EDITBOX *ed, int key, int shifts));

	DIME_METHOD(int, pos_move, (struct EDITBOX *ed, int pos, int select));
	DIME_METHOD(int, pos_left, (struct EDITBOX *ed, int select));
	DIME_METHOD(int, pos_right, (struct EDITBOX *ed, int select));
	DIME_METHOD(int, pos_word_left,
				 (struct EDITBOX *ed, int from_pos, int select));
	DIME_METHOD(int, pos_word_right,
				 (struct EDITBOX *ed, int from_pos, int select));
	DIME_METHOD(int, pos_home, (struct EDITBOX *ed, int select));
	DIME_METHOD(int, pos_end, (struct EDITBOX *ed, int select));

	DIME_METHOD(int, delete_left, (struct EDITBOX *ed));
	DIME_METHOD(int, delete_right, (struct EDITBOX *ed));

	DIME_METHOD(int, toggle_insertmode, (struct EDITBOX *ed));

	DIME_METHOD(int, select_word_at_pos, (struct EDITBOX *ed, int pos));

	DIME_METHOD(int, pos_at_pixel, (struct EDITBOX *ed, int pixel));

	DIME_METHOD(int, clip_cut, (struct EDITBOX *ed));
	DIME_METHOD(int, clip_copy, (struct EDITBOX *ed));
	DIME_METHOD(int, clip_paste, (struct EDITBOX *ed));

	DIME_METHOD(int, set_text, (struct EDITBOX *ed, char *text));

	DIME_METHOD(int, replace_text,
				 (struct EDITBOX *ed, char *text, int start, int length));

	DIME_METHOD(int, num_increase, (struct EDITBOX *ed));
	DIME_METHOD(int, num_decrease, (struct EDITBOX *ed));
	DIME_METHOD(int, num_big_increase, (struct EDITBOX *ed));
	DIME_METHOD(int, num_big_decrease, (struct EDITBOX *ed));

	DIME_METHOD(struct EDITBOX *, create, (struct EDITBOX_OPTIONS *eo, char *text));
	DIME_METHOD(void, destroy, (struct EDITBOX *ed));
} EDITBOX_FUNCTIONS;

/* Special data used by textedit. */
/* Currently unused! */
typedef struct EDITBOX_OPTIONS_TEXT
{
	int available_chars; /* What kind of characters can be used? */
	int force_case;      /* Autoconvert to upper/lower? */
} EDITBOX_OPTIONS_TEXT;

/* Special data used by numedit. */
/* Currently unused! */
typedef struct EDITBOX_OPTIONS_NUM
{
	int decimaltype: 2;  /* Decimals or not? */
	int format     : 2;  /* Long format (long double, long long) or usual? */
	int _case      : 2;  /* Upper or lower case, or both? */
	int force_case : 1;  /* Autoconvert to upper/lower? */
	int plussign, minussign, zerosign;/* What characters to use. */
	int delimitor;       /* Thousand delimitor. */
	int sepdistance;     /* Distance between thousand delimitors. */
	int base;            /* Decimal, hex, oct, bin or something else? */
	int char14, char12, char34; /* What characters to use */
	long double min, max;       /* Smallest/largest possible value. */
	long double smallchange, largechange; /* Up/down arrows increment. */
	long double default_value;  /* What value to use if it's illegal. */
} EDITBOX_OPTIONS_NUM;

/* Special data used by fileedit. */
/* Currently unused! */
typedef struct EDITBOX_OPTIONS_FILE
{
	int system        : 2;  /* What operating system to use. */
	int wildcards     : 2;  /* Enable wildcards? */
	int _case         : 2;  /* Upper or lower case, or both? */
	int force_case    : 1;  /* Autoconvert case? */
	int drive         : 2;  /* Include drive? (DOS/windows only) */
	int directories   : 2;  /* Include directory names? */
	int filename      : 2;  /* Include filename? */
	int extension     : 2;  /* Include extension? */
	int pathdelimitor : 2;  /* Slashes or backslashes? */
} EDITBOX_OPTIONS_FILE;

/* Properties for all kinds of edit boxes. */
typedef struct EDITBOX_OPTIONS
{
	int type;   /* Text, numbers, filename or code? */
	union       /* Extra properties depending on the type of editbox. */
	{
		EDITBOX_OPTIONS_TEXT text;  /* Special data belonging to textedits. */
		/* These are not implemented yet. */
		/* EDITBOX_OPTIONS_NUM  num; */  /* Special data belonging to numedits. */
		/* EDITBOX_OPTIONS_FILE file; */ /* Special data belonging to fileedits. */
	} data;

	int max_length;      /* Limit by number of characters. */
	int max_pixel_width; /* Limit by pixel width. */
	int max_bytes;       /* Limit by length in bytes. */
	FONT *font;          /* What font to use when calculating length. */
	/* int wrap; */      /* Use word wrap? */

	int write_protected; /* OK to change? */
	int dynamic;         /* OK to realloc? */
	int free_text_on_destroy; /* Should destroy_edit() free the text? */
	int default_bytes;   /* Bytes allocated by create_edit(). */
	int increase_bytes;  /* How much to realloc each time? */
	void *extra;         /* May point to a structure with more info. */

	int clipboard_functions;/* OK to select and use clipboard? */
	CLIPBOARD *clipboard;   /* Used by clipboard operations. */
	EDITBOX_FUNCTIONS *funcs;   /* Table with functions to use for editing. */
} EDITBOX_OPTIONS;

/* The edit box struct. */
typedef struct EDITBOX
{
	char *text;             /* The actual text. */
/* long double *val_ld; This is not implemented yet.
	long *val_l; */
	int selstart, sellength;/* Selection, measured in # of characters. */
	int pos;                /* Cursor position, measured in # of characters.*/
	int *insert;            /* Insert or overwrite mode? */

	int alloced_size;       /* How much room do we have? */
	EDITBOX_OPTIONS *options; /* Controls the behavior of the edit box. */
} EDITBOX;

/* Struct with information about the look of an edit box. Not yet used. */
typedef struct GRAPHIC_EDIT
{
	int x1, y1, x2, y2;
	BITMAP *bmp;
	EDITBOX *ed;
	int text_color, bg_color;
	int selection_text_color, selection_bg_color;
	BITMAP *bg_bitmap;
	int cursor_color;
	int cursor_frequence;
	int cursor_blink_mode;
} GRAPHIC_EDIT;


/* Bits in return value from functions. */
#define EDIT_CHANGE_TEXT                  (0x00000001)
#define EDIT_CHANGE_VALUE                 (0x00000002)
#define EDIT_CHANGE_LENGTH                (0x00000004)
#define EDIT_CHANGE_PIXELLENGTH           (0x00000008)
#define EDIT_CHANGE_BYTES                 (0x00000010)
#define EDIT_CHANGE_TEXTOVERFLOWN         (0x00000020)
#define EDIT_CHANGE_TEXTDELETED           (0x00000040)
#define EDIT_CHANGE_TEXTOVERWRITTEN       (0x00000080)
#define EDIT_CHANGE_TEXTLOST              (EDIT_CHANGE_TEXTDELETED |       \
														 EDIT_CHANGE_TEXTOVERWRITTEN |   \
														 EDIT_CHANGE_TEXTOVERFLOWN)
#define EDIT_CHANGE_CLIPBOARD_OPERATION   (0x00000100)
#define EDIT_CHANGE_CLIPBOARD_CONTENTS    (0x00000200)
#define EDIT_CHANGE_CLIPBOARD_LENGTH      (0x00000400)
#define EDIT_CHANGE_CLIPBOARD_PIXELLENGTH (0x00000400)
#define EDIT_CHANGE_CLIPBOARD_BYTES       (0x00000400)
#define EDIT_CHANGE_POS                   (0x00000800)
#define EDIT_CHANGE_PIXELPOS              (0x00001000)
#define EDIT_CHANGE_BYTEPOS               (0x00002000)
#define EDIT_CHANGE_SEL                   (0x00004000)
#define EDIT_CHANGE_INSERT                (0x00008000)
#define EDIT_CHANGE_BEEP                  (0x00010000)
#define EDIT_CHANGE_SHOWCURSOR            (0x00020000)
#define EDIT_CHANGE_WRITEPROTECTED        (0x00040000)
#define EDIT_CHANGE_INVALID_OPERATION     (0x00080000)
#define EDIT_CHANGE_KEYIGNORED            (0x00100000 |                    \
														 EDIT_CHANGE_INVALID_OPERATION)

DIME_FUNC(int, edit_delete_sel, (EDITBOX *ed));
DIME_FUNC(int, edit_select_all, (EDITBOX *ed));
DIME_FUNC(int, install_edit, (void));
DIME_FUNC(int, edit, (EDITBOX *ed, int key, int shifts));
DIME_FUNC(int, edit_set_pos, (EDITBOX *ed, int pos));
DIME_FUNC(int, edit_select_to_pos, (EDITBOX *ed, int pos));
DIME_FUNC(int, edit_set_text, (EDITBOX *ed, char *text));
DIME_FUNC(int, edit_cut, (EDITBOX *ed));
DIME_FUNC(int, edit_copy, (EDITBOX *ed));
DIME_FUNC(int, edit_paste, (EDITBOX *ed));
DIME_FUNC(int, edit_select_word_at_pos, (EDITBOX *ed, int pos));
DIME_FUNC(EDITBOX *, create_edit, (EDITBOX_OPTIONS *eo, char *text));
DIME_FUNC(void, destroy_edit, (EDITBOX *ed));

DIME_FUNC(int, normal_edit, (EDITBOX *ed, int key, int shifts));
DIME_FUNC(int, normal_edit_key_nontext, (EDITBOX *ed, int key, int shifts));
DIME_FUNC(int, normal_edit_replace_text, (EDITBOX *ed, char *text, int start, int length));
DIME_FUNC(int, normal_edit_key_text, (EDITBOX *ed, int c, int shifts));
DIME_FUNC(int, normal_edit_pos_move, (EDITBOX *ed, int pos, int select));
DIME_FUNC(int, normal_edit_pos_left, (EDITBOX *ed, int select));
DIME_FUNC(int, normal_edit_pos_right, (EDITBOX  *ed, int select));
DIME_FUNC(int, normal_edit_pos_word_left, (EDITBOX *ed, int from_pos, int select));
DIME_FUNC(int, normal_edit_pos_word_right, (EDITBOX *ed, int from_pos, int select));
DIME_FUNC(int, normal_edit_pos_home, (EDITBOX *ed, int select));
DIME_FUNC(int, normal_edit_pos_end, (EDITBOX *ed, int select));
DIME_FUNC(int, normal_edit_delete_left, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_delete_right, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_clip_cut, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_clip_paste, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_clip_copy, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_toggle_insertmode, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_select_word_at_pos, (EDITBOX *ed, int pos));
DIME_FUNC(int, normal_edit_pos_at_pixel, (struct EDITBOX *ed, int pixel));
DIME_FUNC(int, normal_edit_set_text, (EDITBOX *ed, char *text));
DIME_FUNC(EDITBOX *, normal_edit_create, (EDITBOX_OPTIONS *eo, char *text));
DIME_FUNC(void, normal_edit_destroy, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_num_increase, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_num_decrease, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_num_big_increase, (EDITBOX *ed));
DIME_FUNC(int, normal_edit_num_big_decrease, (EDITBOX *ed));

DIME_FUNC(int, unix_edit_key_nontext, (EDITBOX *ed, int key, int shifts));

DIME_VAR(int, _edit_insert);
DIME_VAR(int *, _edit_default_insert);

DIME_VAR(CLIPBOARD *, _edit_clipboard);
DIME_VAR(EDITBOX_OPTIONS, _normal_edit_text);
DIME_VAR(EDITBOX_FUNCTIONS, _normal_edit_funcs);

#endif /* if 0 */



/* End of extern "C" { */
#ifdef __cplusplus
	}
#endif

#endif