/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dimeint.h:
	Internal declarations for the Dime library.

	By Sven Sandberg
*/
#ifndef _DIME_INT_H_
#define _DIME_INT_H_


/* Information for one object. */
typedef struct DIALOGF_OBJECT {
	_DIALOGF_FORMAT *type;  /* Pointer into the _dialogf_format list. */
	int num_dialogs;        /* Number of dialog objects created. */
	DIALOG *dialogs;        /* Pointer to first object. */
	int again;              /* Flag telling if we should do another pass. */
	int bottom;             /* Coordinate of bottommost object. */
} DIALOGF_OBJECT;



/* Information set up by `dialogf_init()', needed by `dialogf_run()'. */
typedef struct DIALOGF_DIALOG
{
	DIALOG *dialog;
	int num_formats;

	DIALOGF_OBJECT *objects;

	int old_gui_fg_color, old_gui_mg_color, old_gui_bg_color;
} DIALOGF_DIALOG;



/* The five parts into which one call to dialogf() is divided. */
DIME_FUNC(int, dialogf_run, (DIALOGF_DIALOG *dd));
DIME_FUNC(DIALOGF_DIALOG *, dialogf_start, (char *title, int x_pos, int y_pos, int edit_w, char *format, dime_va_list_p args));
DIME_FUNC(void, dialogf_store_results, (DIALOGF_DIALOG *dd));
DIME_FUNC(void, dialogf_reset_dialog, (DIALOGF_DIALOG *dd));
DIME_FUNC(void, dialogf_end, (DIALOGF_DIALOG *dd));



/* Registers a new format specifier. */
DIME_FUNC(void, register_dialogf_format, (
 char *specifier,
 int handle_desc,
 DIME_METHOD(int, count_dialog, (char *text)),
 DIME_METHOD(int, create_dialog, (DIALOG *dialog, char *desc, char *arg_text, dime_va_list_p args)),
 DIME_METHOD(int, outside_dialog, (DIALOG *object)),
 DIME_METHOD(void, store_dialog_result, (DIALOG *object)),
 DIME_METHOD(void, reset_dialog, (DIALOG *object)),
 DIME_METHOD(void, destroy_dialog_data, (DIALOG *object))));

/* Exactly as `vdialogf()', except that it takes a pointer to a `va_list'
	rather than just a `va_list'. */
DIME_FUNC(int, _vdialogf, (char *title, int x_pos, int y_pos, int edit_w, char *format, dime_va_list_p args));

/* Internal hardware colors. */
DIME_VAR(int, _dime_text_color);
DIME_VAR(int, _dime_disabled_text_color);
DIME_VAR(int, _dime_error_text_color);
DIME_VAR(int, _dime_border_color);
DIME_VAR(int, _dime_background_color);
DIME_VAR(int, _dime_edit_field_color);
DIME_VAR(int, _dime_button_color);
DIME_VAR(int, _dime_dark_shadow_color);
DIME_VAR(int, _dime_shadow_color);
DIME_VAR(int, _dime_highlight_color);
DIME_VAR(int, _dime_light_highlight_color);

/* Internal actual pointers. */
DIME_VAR(FONT *, _dime_font);
DIME_VAR(FONT *, _dime_title_font);
DIME_VAR(FONT *, _dime_button_font);
DIME_VAR(BITMAP *, _dime_bmp);
DIME_VAR(int, _dime_screen_w);
DIME_VAR(int, _dime_screen_h);

/* Some information that the creation functions may need. */
DIME_VAR(DIALOG *, _dime_dialog);
DIME_VAR(int, _dialogf_final_pass);
DIME_VAR(int, _dialogf_pass_index);

/* Helper for drawing an inwards bevel. */
DIME_FUNC(void, inwards_bevel, (DIALOG *d, int ofs, int white_space));

/* Helper for creating a description text. */
DIME_FUNC(void, dialogf_create_desc, (DIALOG *d, int y, char *desc));

/* Helper for setting up stuff like _dime_bmp and the like. */
DIME_FUNC(void, dime_setup_global_stuff, (void));

/* Helpers for getting arguments from argument lists. */
#define DIME_ARG_COMMA     (0x0001)
#define DIME_ARG_END       (0x0002)
#define DIME_ARG_NONDIGIT  (0x0004)
#define DIME_ARG_DIGIT     (0x0008)
#define DIME_ARG_READ      (0x0010)
DIME_FUNC(int, dime_get_int_arg, (char **arg, int *out));
DIME_FUNC(int, dime_get_double_arg, (char **arg, double *out));
DIME_FUNC(int, dime_get_string_arg, (char **arg, char *out, int max_size));
DIME_FUNC(int, dime_get_strlist_arg, (char **arg_text, int *out_num, char ***out_list));

/* Helper function for calculator edit boxes. */
DIME_FUNC(double, dime_evaluate, (char *equation, int *error));

/* Helpers for clamping the result of %cint & co. */
DIME_FUNC(double, dialogf_clamp_calc, (DIALOG *d, double min, double max, int round));
DIME_FUNC(double, dialogf_clamp_plain, (DIALOG *d, double min, double max, int round));

/* Returns nonzero if x is NaN, zero otherwise. */
DIME_FUNC(int, dime_isnan, (double x));


#ifdef DEBUGMODE
	#define DIME_ASSERT(cond, msg)                                           \
		(                                                                     \
			(cond) ?                                                           \
				(void)0                                                         \
			:                                                                  \
			(                                                                  \
				al_trace("DIME_ASSERT failed in %s.%d:\n",                      \
							__FILE__, __LINE__),                                   \
				al_trace("The condition was: (%s)\n", #cond),                   \
				al_trace msg ,                                                  \
				al_assert(__FILE__, __LINE__),                                  \
				(void)0                                                         \
			)                                                                  \
		)
#else
	#define DIME_ASSERT(cond, msg)
#endif


#define DIME_MALLOC(nelems, TYPE)     ((TYPE *)malloc((nelems) * sizeof(TYPE)))
#define DIME_REALLOC(p, nelems, TYPE) ((TYPE *)realloc((p), (nelems) * sizeof(TYPE)))


#define DIME_UNUSED_ARG(arg)  (void)arg;



#endif