#   ___  _
#  |   \(_)_ __  ___
#  | |) | | '  \/ -_)
#  |___/|_|_|_|_\___|
#
#  makefile.wat:
#  Makefile for the Dime library with Watcom. Should not be used directly!
#
#  By Sven Sandberg
#
#  See readme.txt for more information about Dime.
#
#  See makefile.all for a list of the available targets.
#
#
#  This build uses djgpp for building the assembler sources and calculating
#  source dependencies, so you'll need to have that installed as well.
#
#  The "depend" target and assembler operations use sed.



# -------- Define some variables that the primary makefile may use. --------

PLATFORM = Watcom

EXE_SUFFIX = .exe
OBJ_SUFFIX = .obj
LIB_SUFFIX = .lib

HTML_SUFFIX = .htm
INFO_SUFFIX = .inf
TEXI_SUFFIX = .txi
_TX_HTML_FLAG = -htm
_TX_TEXI_FLAG = -txi

ifndef OBJ_ROOT_DIR
OBJ_ROOT_DIR = obj
OBJ_ROOT_DIR_D = obj
endif
OBJ_PLATFORM_ROOT_DIR = $(OBJ_ROOT_DIR)/watcom
OBJ_PLATFORM_ROOT_DIR_D = $(OBJ_ROOT_DIR_D)\watcom
OBJ_DIR = $(OBJ_PLATFORM_ROOT_DIR)/$(PACKAGE5)
OBJ_DIR_D = $(OBJ_PLATFORM_ROOT_DIR_D)\$(PACKAGE5)

ifndef LIB_ROOT_DIR
LIB_ROOT_DIR = lib
LIB_ROOT_DIR_D = lib
endif
LIB_DIR = $(LIB_ROOT_DIR)/watcom
LIB_DIR_D = $(LIB_ROOT_DIR)\watcom

LIB_BASENAME = $(PACKAGE5).lib
LIB_NAME = $(LIB_DIR)/$(LIB_BASENAME)



# -------- Check that the WATCOM environment variable is set. --------

.PHONY: badwatcom

ifndef WATCOM
badwatcom:
	@echo Your WATCOM environment variable is not set!
	@echo You must set the environment variable WATCOM to point to your Watcom
	@echo directory, eg "set WATCOM=c:\watcom"
endif

WATCOMDIR_U = $(subst \,/,$(WATCOM))
WATCOMDIR_D = $(subst /,\,$(WATCOM))

SYSTEM_INCLUDE_DIR = $(WATCOMDIR_U)/h
SYSTEM_INCLUDE_DIR_D = $(WATCOMDIR_D)\h
SYSTEM_LIB_DIR = $(WATCOMDIR_U)/lib386
SYSTEM_LIB_DIR_D = $(WATCOMDIR_D)\lib386



# -------- Try to figure out the compiler version. --------

ifndef WATCOM_VERSION
ifeq ($(wildcard $(WATCOMDIR_U)/binw/wdisasm.exe),)
WATCOM_VERSION=11
else
WATCOM_VERSION=10.6
endif
endif



# -------- Find Allegro, runner and makedoc. --------

NEED_ALLEGRO = 1

.PHONY: badalleg

ifdef ALLEGRO
ALLEGRODIR_U = $(subst \,/,$(ALLEGRO))
else
badalleg:
	@echo Couldnt find Allegro!
	@echo Is your ALLEGRO environment variable set correctly? It should
	@echo point to the place where Allegro lives, eg c:\watcom\allegro.
endif


RUNNER = $(wildcard $(ALLEGRODIR_U)/obj/watcom/runner.exe)
ifeq ($(RUNNER),)
.PHONY: badrunner
badrunner:
	@echo Couldnt find Allegros runner utility.
	@echo Have you forgotten to make Allegro? You need to make the Watcom version of
	@echo Allegro before making the Watcom version of Dime.
else
endif


ifdef NEED_MAKEDOC
.PHONY: badmakedoc

ifneq ($(wildcard $(ALLEGRODIR_U)/docs/makedoc$(EXE_SUFFIX)),)
MAKEDOC_PROGRAM := $(ALLEGRODIR_U)/docs/makedoc$(EXE_SUFFIX)
else
badmakedoc:
	@echo Couldnt find Allegros makedoc utility.
	@echo Have you forgotten to make Allegro? You need to make Allegro, or at least
	@echo its documentation, before making Dime.
endif

endif # ifdef NEED_MAKEDOC



# -------- give a sensible default target for make without any args --------

.PHONY: _default

_default: default



# -------- decide what compiler options to use --------

ifdef WARNMODE
WFLAGS = -w3 -we
else
WFLAGS = -w1
endif

ifdef DEBUGMODE
# -------- debugging build --------
CFLAGS = -DDEBUGMODE=$(DEBUGMODE) $(WFLAGS) -bt=dos4g -5s -s -d2
SFLAGS = -DDEBUGMODE=$(DEBUGMODE) -Wall
LFLAGS = "option quiet" "option stack=128k" "system dos4g" "debug all"
else
ifdef PROFILEMODE
# -------- profiling build --------
CFLAGS = $(WFLAGS) -bt=dos4g -5s -ox -et
SFLAGS = -Wall
LFLAGS = "option quiet" "option stack=128k" "system dos4g"
else
# -------- optimised build --------
CFLAGS = $(WFLAGS) -bt=dos4g -5s -ox
SFLAGS = -Wall
LFLAGS = "option quiet" "option stack=128k" "system dos4g"
LIB_FLAGS = "library alleg"
endif
endif

CFLAGS_NO_OPTIMIZE = $(WFLAGS) -bt=dos4g -5s -s

COMPILE_FLAGS = $(subst src/,-dDIME_SRC ,$(findstring src/, $<))$(CFLAGS)
COMPILE_FLAGS_NO_OPTIMIZE = $(subst src/,-dDIME_SRC ,$(findstring src/, $<))$(CFLAGS_NO_OPTIMIZE)



# -------- List platform specific objects and programs. --------

VPATH +=

OBJ_LIST = $(COMMON_OBJS)

# Insert names of targets to build platform specific programs here.
PROGRAMS =
# Insert targets to build platform specific programs here.
# myprogram: path/myprogram$(EXE_SUFFIX)



# -------- Define how to compile. --------

GCC2WATCOM = -D__WATCOMC__ -D__SW_3S -D__SW_S -U__GNUC__ -UDJGPP -U__unix__

COMPILE_C_TO_OBJ_DEPS =
define COMPILE_C_TO_OBJ
$(RUNNER) wcc386 \\ $< @ $(COMPILE_FLAGS) -zq -fr=nul -I./include -fo=$@
endef
define COMPILE_C_TO_OBJ_NO_OPTIMIZE
$(RUNNER) wcc386 \\ $< @ $(COMPILE_FLAGS_NO_OPTIMIZE) -zq -fr=nul -I./include -fo=$@
endef

COMPILE_S_TO_OBJ_DEPS =
ifeq ($(WATCOM_VERSION),11)
# Watcom 11.0 supports COFF.
define COMPILE_S_TO_OBJ
gcc $(GCC2WATCOM) $(SFLAGS) -I./include -x assembler-with-cpp -o $(OBJ_DIR)/$*.obj -c $<
endef
else
# Black magic to build asm sources with Watcom 10.6.
define COMPILE_S_TO_OBJ
gcc $(GCC2WATCOM) $(SFLAGS) -I./include -x assembler-with-cpp -o $(OBJ_DIR)/$*.o -c $<
$(RUNNER) wdisasm \\ -a $(OBJ_DIR)/$*.o > $(OBJ_DIR)/$*.lst
sed -e "s/\.text/_TEXT/; s/\.data/_DATA/; s/\.bss/_BSS/; s/\.386/\.586/; s/lar *ecx,cx/lar ecx,ecx/" $(OBJ_DIR)/$*.lst > $(OBJ_DIR)/$*.asm
$(RUNNER) wasm \\ $(WFLAGS) -zq -fr=nul -fp3 -fo=$@ $(OBJ_DIR)/$*.asm
endef
endif

LINK_OBJ_TO_LIB_DEPS =
define LINK_OBJ_TO_LIB
$(RUNNER) wlib \\ @ -q -b -n $(LIB_NAME) $(addprefix +,$(LIB_OBJS))
endef

LINK_OBJ_TO_EXE_DEPS =
define LINK_OBJ_TO_EXE
$(RUNNER) wlink \\ @ $(LFLAGS) "name $@" "file $<" "library $(LIB_NAME)" $(LIB_FLAGS)
endef

LINK_ALL_OBJ_TO_EXE_DEPS =
define LINK_ALL_OBJ_TO_EXE
$(RUNNER) wlink \\ @ $(LFLAGS) "name $@" "file $^" "library $(LIB_NAME)" $(LIB_FLAGS)
endef



# -------- Rules for installing and removing the library files. --------

$(SYSTEM_LIB_DIR)/$(LIB_BASENAME): $(LIB_NAME)
	copy $(subst /,\,$< $@)

$(SYSTEM_INCLUDE_DIR)/dime.h: include/dime.h
	copy $(subst /,\,$< $@)

$(SYSTEM_INCLUDE_DIR)/dime:
	md $(subst /,\,$@)

$(SYSTEM_INCLUDE_DIR)/dime/%.h: include/dime/%.h include/dime
	copy $(subst /,\,$< $@)


INSTALL_HEADERS = $(addprefix $(SYSTEM_INCLUDE_DIR)/dime/,$(notdir $(wildcard include/dime/*.h)))

INSTALL_FILES = $(SYSTEM_LIB_DIR)/$(LIB_BASENAME) \
					 $(SYSTEM_INCLUDE_DIR)/dime.h \
					 $(SYSTEM_INCLUDE_DIR)/dime \
					 $(INSTALL_HEADERS)

install: $(INSTALL_FILES)
	@echo The $(DESCRIPTION) Watcom library has been installed.

UNINSTALL_FILES = $(SYSTEM_LIB_DIR)/dime.lib $(SYSTEM_LIB_DIR)/dimed.lib \
						$(SYSTEM_INCLUDE_DIR)/dime.h \
						$(SYSTEM_INCLUDE_DIR)/dime/*.h

uninstall:
	-rm -fv $(UNINSTALL_FILES)
	-rd $(SYSTEM_INCLUDE_DIR_D)\dime
	@echo All gone!



# -------- Compile anything that can't be done in makefile.all. --------

# this section is empty



# -------- Generate automatic dependencies. --------
# Note that this must be done with gcc!

DEPEND_PARAMS = $(GCC2WATCOM) -MM -MG -I./include -DSCAN_DEPEND

depend:
	gcc $(DEPEND_PARAMS) src/*.c examples/*.c > _depend.tmp
	sed -e "s/^[a-zA-Z0-9_\/]*\///" -e "s/[a-zA-Z0-9_\/.:-]*version.h//" _depend.tmp > _depend2.tmp
	sed -e "s/^\([a-zA-Z0-9_]*\.o:\)/obj\/watcom\/dime\/\1/" _depend2.tmp > obj/watcom/dime/makefile.dep
	sed -e "s/^\([a-zA-Z0-9_]*\.o:\)/obj\/watcom\/dimed\/\1/" _depend2.tmp > obj/watcom/dimed/makefile.dep
	rm _depend.tmp _depend2.tmp