/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	ddialogf.c:
	Dialogf functions for editing booleans as check boxes.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	int my_int;
	dialogf(..., "Description%dialogf[Open dialog]",
			  "Title of sub-dialog", x, y, edit_w,
			  "New format string%bool[]", &my_int);
*/
#include <stdarg.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_dialogf_ex:
	Worker function for creating a dialogf() object. Does the work for both
	formats.
todo: remove end_proc
*/
static void dialogf_create_dialogf_ex(DIALOG *d, char *arg_text,
												  dime_va_list_p args, int v,
												  int end_proc)
{
	char *title;
	int x, y, w;
	char *format;
	int state;
	char buf[4096];
	/* Watcom uses the definition char *va_list[] so it can't be an lvalue. */
	void *sub_args;

	/* Parse parameters. */
	state = dime_get_string_arg(&arg_text, buf, sizeof(buf));
	DIME_ASSERT(state & DIME_ARG_END,
					("Too many arguments in format modifier for %%dialogf."));

	d->proc = d_dime_button_proc;
	d->flags |= D_EXIT;
	d->h += 6;
	d->dp = ustrdup(buf);

// todo: think this should be removed
// if (end_proc)
//    /* `int (*)(DIALOG *, int, int)' doesn't work, must use `void *'. */
//    d->dp2 = dime_va_arg_p(args, void *);
// else
		d->dp2 = NULL;

	title = dime_va_arg_p(args, char *);
	x = dime_va_arg_p(args, int);
	y = dime_va_arg_p(args, int);
	w = dime_va_arg_p(args, int);
	format = dime_va_arg_p(args, char *);

	if (v) {
		sub_args = dime_va_arg_p(args, void *);
		d->dp3 = dialogf_start(title, x, y, w, format,
									  (dime_va_list_p)&sub_args);
	}
	else
		d->dp3 = dialogf_start(title, x, y, w, format, args);
}



/* dialogf_create_dialogf:
*/
int dialogf_create_dialogf(DIALOG *d, char *desc, char *arg_text,
									dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	dialogf_create_dialogf_ex(d, arg_text, args, FALSE, FALSE);
	return 0;
}



/* dialogf_create_vdialogf:
*/
int dialogf_create_vdialogf(DIALOG *d, char *desc, char *arg_text,
									 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	dialogf_create_dialogf_ex(d, arg_text, args, TRUE, FALSE);
	return 0;
}



/* dialogf_create_dialogf2:
*/
int dialogf_create_dialogf2(DIALOG *d, char *desc, char *arg_text,
									 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	dialogf_create_dialogf_ex(d, arg_text, args, FALSE, TRUE);
	return 0;
}



/* dialogf_create_vdialogf2:
*/
int dialogf_create_vdialogf2(DIALOG *d, char *desc, char *arg_text,
									  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	dialogf_create_dialogf_ex(d, arg_text, args, TRUE, TRUE);
	return 0;
}



/* dialogf_outside_dialogf:
*/
int dialogf_outside_dialogf(DIALOG *d)
{
	int result;

	result = dialogf_run((DIALOGF_DIALOG *)d->dp3);

// todo: don't think this should be here
// if (d->dp2)
//    result = ((int (*)(DIALOG *, int, int))d->dp2)(d->dp3, result);

	if (result == 1)
		dialogf_store_results((DIALOGF_DIALOG *)d->dp3);
	else
		dialogf_reset_dialog((DIALOGF_DIALOG *)d->dp3);

	return 0;
}



/* dialogf_store_dialogf:
*/
void dialogf_store_dialogf(DIALOG *d)
{
	dialogf_store_results((DIALOGF_DIALOG *)d->dp3);
}



/* dialogf_reset_dialogf:
*/
void dialogf_reset_dialogf(DIALOG *d)
{
	dialogf_reset_dialog((DIALOGF_DIALOG *)d->dp3);
}



/* dialogf_destroy_dialogf:
*/
void dialogf_destroy_dialogf(DIALOG *d)
{
	dialogf_end((DIALOGF_DIALOG *)d->dp3);
	free(d->dp);
}