/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dfilename.c:
	Dialogf functions for editing filenames.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	char filename[1024];
	dialogf(..., "Select a file%filename[512,txt,Select a text file]",
			  filename);
*/
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* This was introduced in a late version of Allegro, so I'll keep it
	optional. */
#undef HAS_FSEL_HOOKS
#if ALLEGRO_DATE >= 20010401
	#define HAS_FSEL_HOOKS
#endif



/* dialogf_create_filename:
*/
int dialogf_create_filename(DIALOG *d, char *desc, char *arg_text,
									 dime_va_list_p args)
{
	int state;
	char buf[1024];
	char *ext, *title;
	int button_w;

	/* Parse parameters. */
	state = dime_get_int_arg(&arg_text, &d->d1);
	DIME_ASSERT(state & DIME_ARG_READ,
					("No max length specified for %%filename."));
	DIME_ASSERT(state & DIME_ARG_COMMA,
					("',' expected in modifier for %%filename: [%s]", arg_text));

	state = dime_get_string_arg(&arg_text, buf, sizeof(buf));
	DIME_ASSERT(state & DIME_ARG_COMMA,
					("',' expected in modifier for %%filename: [%s]", arg_text));
	if (state & DIME_ARG_READ)
		ext = ustrdup(buf);
	else
		ext = NULL;

	state = dime_get_string_arg(&arg_text, buf, sizeof(buf));
	DIME_ASSERT(state & DIME_ARG_END,
					("Garbage found at end of format modifier for %%filename: "
					 "[%s]",
					 arg_text));
	if (state & DIME_ARG_READ)
		title = ustrdup(buf);
	else
		title = ustrdup(desc);

	/* Create text editor. */
	d->proc = d_dime_edit_proc;
	button_w = text_length(_dime_font, uconvert_ascii("...", buf)) + 6;
	d->w -= button_w;
	d->h += 6;
	d->fg = _dime_text_color;
	d->bg = _dime_edit_field_color;
	d->dp = DIME_MALLOC(d->d1 * uwidth_max(U_CURRENT) + ucwidth(0), char);
	d->dp3 = dime_va_arg_p(args, char *);
	d->dp2 = _dime_font;

	/* Create button for opening file selector. */
	d++;
	d->proc = d_dime_button_proc;
	d->x = d->x + d->w - button_w;
	d->w = button_w;
	d->h += 6;
	d->fg = _dime_text_color;
	d->bg = _dime_button_color;
	d->flags |= D_EXIT;
	d->dp = ustrdup(uconvert_ascii("...", buf));
	d->dp2 = title;
	d->dp3 = ext;

	return 0;
}



/* fsel_callback:
	Function that will be called from Allegro's file selector in place of
	d_shadow_box_proc().
*/
#ifdef HAS_FSEL_HOOKS
static int (*old_shadow_box_proc)(int msg, DIALOG *d, int c);
static int fsel_callback(int msg, DIALOG *d, int c)
{
	int ret;

	if (old_shadow_box_proc == NULL)
		ret = d_shadow_box_proc(msg, d, c);
	else
		ret = old_shadow_box_proc(msg, d, c);

	if ((msg == MSG_IDLE) && (ret == D_O_K))
		if (dialogf_callback)
			dialogf_callback(d);

	return ret;
}
#endif



/* dialogf_outside_filename:
	This is called after the dialog has temporarily been shut down.
*/
int dialogf_outside_filename(DIALOG *d)
{
	char *buf;
	char *source, *dest;
	int i, c;
	int size;
	BITMAP *bmp;

	bmp = create_bitmap(_dime_screen_w, _dime_screen_h);
	if (bmp != NULL) {
		scare_mouse();
		blit(_dime_bmp, bmp, 0, 0, 0, 0, _dime_screen_w, _dime_screen_h);
		unscare_mouse();
	}

	size = d[-1].d1 * uwidth_max(U_CURRENT) + ucwidth(0);
	/* The buffer in fsel.c is 512 bytes. */
	size = MAX(size, 512);
	buf = DIME_MALLOC(size, char);
	ustrcpy(buf, d[-1].dp);

	#ifdef HAS_FSEL_HOOKS
		old_shadow_box_proc = gui_shadow_box_proc;
		gui_shadow_box_proc = fsel_callback;
	#endif

	if (file_select(d->dp2, buf, d->dp3)) {
		source = buf;
		dest = d[-1].dp;
		/* Copy only a limited number of characters. */
		for (c = ugetx(&source), i = 0;
			  c && (i < d[-1].d1);
			  c = ugetx(&source), i++)
			dest += usetc(dest, c);
		usetc(dest, 0);
	}
	free(buf);

	#ifdef HAS_FSEL_HOOKS
		gui_shadow_box_proc = old_shadow_box_proc;
	#endif

	if (bmp == NULL)
		return 1;
	scare_mouse();
	blit(bmp, _dime_bmp, 0, 0, 0, 0, _dime_bmp->w, _dime_bmp->h);
	unscare_mouse();
	destroy_bitmap(bmp);
	return 0;
}



/* dialogf_store_filename:
*/
void dialogf_store_filename(DIALOG *d)
{
	/* As long as nothing on the way is buggy, we can assume that there is no
		more characters than specified. */
	ustrcpy((char *)d->dp3, (char *)d->dp);
}



/* %filename[] uses dialogf_reset_string() */



/* dialogf_destroy_filename:
*/
void dialogf_destroy_filename(DIALOG *d)
{
	free(d->dp);
	free(d[1].dp);
	free(d[1].dp2);
	free(d[1].dp3);
}