/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dpfloat.c:
	Functions for all calculator floating point formats, ie %float and
	%double.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	float my_float;
	dialogf(..., "Select a number:%float[]", &my_float);

	double my_double;
	dialogf(..., "Select a number:%double[]", &my_double);
*/
#include <math.h>
#include <stdio.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_floatingpoint:
*/
int dialogf_create_floatingpoint(DIALOG *d, char *arg_text,
											dime_va_list_p args, int is_double)
{
	char buf[256];
	int i;
	int w, max_width, widest_digit, exponent_width;
	DIALOGF_EDIT_NUMBER *n;

	/* First parse format and set up edit box just like for the plain types.
	*/
	dialogf_create_pfloatingpoint(d, arg_text, args, is_double);
	n = (DIALOGF_EDIT_NUMBER *)d->dp2;
	d->proc = d_double_calc_edit_proc;
	d++;

	/* Find max length of result. */
	widest_digit = 0;
	for (i = '0'; i <= '9'; i++) {
		w = dime_char_width(_dime_font, i);
		if (w > widest_digit)
			widest_digit = w;
	}

	max_width = widest_digit * 10 + dime_char_width(_dime_font, '.');
	if (n->min_val < 0)
		max_width += dime_char_width(_dime_font, '-');
	exponent_width = 0;
	if (n->min_val < -999990000) {
		sprintf(buf, "%.0f", n->min_val);
		if ((n->min_val == -HUGE_VAL) || (strlen(buf) > 11))
			exponent_width = text_length(_dime_font, "e+");
	}
	else if (n->max_val > 999990000) {
		sprintf(buf, "%.0f", n->max_val);
		if ((n->max_val == HUGE_VAL) || (strlen(buf) > 11))
			exponent_width = text_length(_dime_font, "e+");
	}
	if ((n->min_val < 0.0001) &&
		 (n->max_val > -0.0001)) {
		if (text_length(_dime_font, "e-") > exponent_width)
			exponent_width = text_length(_dime_font, "e-");
	}
	if (exponent_width > widest_digit)
		max_width += exponent_width - widest_digit;

	/* Set up result object. */
	d->proc = d_calc_edit_result_proc;
	d->x = d[-1].x + d[-1].w + 6;
	d->y = d[-1].y + 3;
	d->w = max_width;
	d->fg = _dime_text_color;
	d->bg = _dime_background_color;
	d->dp = DIME_MALLOC(256 * uwidth_max(U_CURRENT) + ucwidth(0), char);
	d->dp2 = _dime_font;

	return 0;
}



/* dialogf_create_double:
*/
int dialogf_create_double(DIALOG *d, char *desc, char *arg_text,
								  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_floatingpoint(d, arg_text, args, TRUE);
}



/* dialogf_store_double:
*/
void dialogf_store_double(DIALOG *d)
{
	*(double *)d->dp3 = dialogf_clamp_calc(d, -HUGE_VAL, HUGE_VAL, FALSE);
}



/* dialogf_create_float:
*/
int dialogf_create_float(DIALOG *d, char *desc, char *arg_text,
								 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_floatingpoint(d, arg_text, args, FALSE);
}



/* dialogf_store_float:
*/
void dialogf_store_float(DIALOG *d)
{
	*(float *)d->dp3 = dialogf_clamp_calc(d, -HUGE_VAL, HUGE_VAL, FALSE);
}