/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dime.c:
	Some global variables that are common to dialogf() and message_box().

	By Sven Sandberg

	See readme.txt for more information about Dime.
*/
#include <allegro.h>
#include <allegro/aintern.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



_DIALOGF_FORMAT *_dialogf_formats = NULL;



/* Colors of different parts of the dialog. */
RGB dime_text_rgb            = { 0,   0,   0,   0 },
	 dime_disabled_text_rgb   = { 128, 128, 128, 0 },
	 dime_error_text_rgb      = { 255, 0,   0,   0 };
RGB dime_border_rgb          = { 0,   0,   0,   0 };
RGB dime_background_rgb      = { 192, 192, 192, 0 },
	 dime_edit_field_rgb      = { 255, 255, 255, 0 },
	 dime_button_rgb          = { 192, 192, 192, 0 };
RGB dime_dark_shadow_rgb     = { 0,   0,   0,   0 },
	 dime_shadow_rgb          = { 128, 128, 128, 0 },
	 dime_highlight_rgb       = { 192, 192, 192, 0 },
	 dime_light_highlight_rgb = { 255, 255, 255, 0 };

/* Screen bitmap to draw on. */
BITMAP *dime_bmp = NULL;

/* Fonts to be used in dialogs. */
FONT *dime_font = NULL, *dime_title_font = NULL, *dime_button_font = NULL;

/* Clear this flag if you don't want to yield timeslices. */
int dime_yield = TRUE;
/* Clear this flag if you don't want the dialog to clean up the background
	after it has finished. */
int dime_clean_screen = TRUE;

/* Window measurement variables. */
int dime_window_border_thickness = 5;
int dime_window_title_spacing = 10;
int dime_window_button_spacing = 10,
	 dime_window_between_button_spacing = 10;
int dime_window_description_spacing = 5;
int dime_window_line_spacing = 5;
int dime_window_button_w = 60, dime_window_button_h = 20;



/* Internal stuff. */
int _dime_text_color, _dime_disabled_text_color, _dime_error_text_color;
int _dime_border_color;
int _dime_background_color, _dime_edit_field_color, _dime_button_color;
int _dime_dark_shadow_color, _dime_shadow_color,
	 _dime_highlight_color, _dime_light_highlight_color;

FONT *_dime_font, *_dime_title_font, *_dime_button_font;
BITMAP *_dime_bmp;
int _dime_screen_w, _dime_screen_h;



/* dime_setup_global_stuff:
	Sets up some global stuff that both `message_box()' and `dialogf()' needs.
*/
void dime_setup_global_stuff(void)
{
	RGB_MAP *old_rgb_map = NULL;

	if (dime_bmp == NULL) {
		_dime_bmp = screen;
		_dime_screen_w = SCREEN_W;
		_dime_screen_h = SCREEN_H;
	}
	else {
		_dime_bmp = dime_bmp;
		_dime_screen_w = dime_bmp->w;
		_dime_screen_h = dime_bmp->h;
	}

	if (dime_font == NULL)
		_dime_font = font;
	else
		_dime_font = dime_font;

	if (dime_title_font == NULL)
		_dime_title_font = font;
	else
		_dime_title_font = dime_title_font;

	if (dime_button_font == NULL)
		_dime_button_font = font;
	else
		_dime_button_font = dime_button_font;

	/* Convert from rgb to hardware color. */
	old_rgb_map = rgb_map;
	rgb_map = NULL;
	#define FIX_COLOR(name) \
	 (_dime_##name##_color = makecol(dime_##name##_rgb.r,                    \
												dime_##name##_rgb.g,                    \
												dime_##name##_rgb.b))
	FIX_COLOR(text);
	FIX_COLOR(disabled_text);
	FIX_COLOR(error_text);
	FIX_COLOR(background);
	FIX_COLOR(edit_field);
	FIX_COLOR(button);
	FIX_COLOR(dark_shadow);
	FIX_COLOR(shadow);
	FIX_COLOR(highlight);
	FIX_COLOR(light_highlight);
	FIX_COLOR(border);
	#undef FIX_COLOR
	rgb_map = old_rgb_map;

	gui_fg_color = _dime_text_color;
	gui_bg_color = _dime_background_color;
	gui_mg_color = _dime_shadow_color;
}



/* return_1:
	Dummy function used by formats that always create one object.
*/
int dialogf_return_1(char *text)
{
	DIME_UNUSED_ARG(text);
	return 1;
}



/* return_2:
	Dummy function used by formats that always create two objects.
*/
int dialogf_return_2(char *text)
{
	DIME_UNUSED_ARG(text);
	return 2;
}



/* dialogf_destroy_dp:
*/
void dialogf_destroy_dp(DIALOG *d)
{
	free(d->dp);
}



/* register_dialogf_format:
	Registers a new format specifier.
*/
void register_dialogf_format(
 char *specifier,
 int handle_desc,
 int (*count_dialog)(char *text),
 int (*create_dialog)(DIALOG *dialog, char *desc, char *arg_text,
							 dime_va_list_p args),
 int (*outside_dialog)(DIALOG *object),
 void (*store_dialog_result)(DIALOG *object),
 void (*reset_dialog)(DIALOG *object),
 void (*destroy_dialog_data)(DIALOG *object))
{
	_DIALOGF_FORMAT f;
	_DIALOGF_FORMAT *iter;

	f.specifier = specifier;
	f.handle_desc = handle_desc;
	f.count_dialog = count_dialog;
	f.create_dialog = create_dialog;
	f.outside_dialog = outside_dialog;
	f.store_dialog_result = store_dialog_result;
	f.reset_dialog = reset_dialog;
	f.destroy_dialog_data = destroy_dialog_data;
	f.next = _dialogf_formats;

	/* Don't add already added formats: it is valid to add a format twice. */
	for (iter = _dialogf_formats; iter != NULL; iter = iter->next)
		if (strcmp(specifier, iter->specifier) == 0)
			return;

	/* Insert as head. (`next' pointer is already set up.) */
	_dialogf_formats = DIME_MALLOC(1, _DIALOGF_FORMAT);
	*_dialogf_formats = f;
}



/* dime_init:
	Registers all dialogf formats.
*/
int dime_init(void)
{
	int i;

	for (i = 0; _dialogf_format_list[i].specifier != NULL; i++)
		register_dialogf_format(_dialogf_format_list[i].specifier,
										_dialogf_format_list[i].handle_desc,
										_dialogf_format_list[i].count_dialog,
										_dialogf_format_list[i].create_dialog,
										_dialogf_format_list[i].outside_dialog,
										_dialogf_format_list[i].store_dialog_result,
										_dialogf_format_list[i].reset_dialog,
										_dialogf_format_list[i].destroy_dialog_data);

	_add_exit_func(dime_exit);

	return 0;
}



/* dime_exit:
	Frees the memory allocated by register_dialogf_format().
*/
void dime_exit(void)
{
	_DIALOGF_FORMAT *iter, *next;

	for (iter = _dialogf_formats; iter != NULL; iter = next) {
		next = iter->next;
		free(iter);
	}
	_dialogf_formats = NULL;

	_remove_exit_func(dime_exit);
}