/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dinteger.c:
	Functions for all calculator integer formats, ie
	%int, %uint, %short, %ushort, %char, %uchar, %longlong, %ulonglong

	By Sven Sandberg

	See readme.txt for more information about Dime.


	int my_int = 0;
	dialogf(..., "Select a number between -4711 and 471100:%int[-4711,471100]",
			  &my_int);

	unsigned int my_uint = 0;
	dialogf(..., "Select a positive number:%uint[,]",
			  &my_uint);

	short my_short = 0;
	dialogf(..., "Select a number between -4711 and 4711:%short[-4711,4711]",
			  &my_short);

	unsigned short my_ushort = 0;
	dialogf(..., "Select a number between 0 and 4711:%ushort[,4711]",
			  &my_ushort);

	char my_char = 0;
	dialogf(..., "Select a number between -47 and 11:%char[-47,11]",
			  &my_char);

	unsigned char my_uchar = 0;
	dialogf(..., "Select a number between 11 and 47:%uchar[11,47]",
			  &my_uchar);
*/
#include <limits.h>
#include <stdio.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_integer:
*/
int dialogf_create_integer(DIALOG *d, char *arg_text, dime_va_list_p args,
									double min_val, double max_val)
{
	char buf[256];
	int i;
	int w, widest_digit;
	int result_max_len, len;
	DIALOGF_EDIT_NUMBER *n;

	/* First parse format and set up edit box just like for the plain types.
	*/
	dialogf_create_pinteger(d, arg_text, args, min_val, max_val);
	n = (DIALOGF_EDIT_NUMBER *)d->dp2;
	d->proc = d_int_calc_edit_proc;
	d++;

	/* Find max length of result. */
	widest_digit = 0;
	for (i = '0'; i <= '9'; i++) {
		w = dime_char_width(_dime_font, i);
		if (w > widest_digit)
			widest_digit = w;
	}
	w = dime_char_width(_dime_font, '-');
	if (w > widest_digit)
		widest_digit = w;

	/* long long is a bit tricky to implement and doesn't work yet. */
	#ifdef DIME_LONG_LONG
		if (min_val < -1)
			sprintf(buf, "%Ld", (LONG_LONG)n->max_val);
		else
			sprintf(buf, "%Lu", (unsigned LONG_LONG)n->max_val);
		result_max_len = strlen(buf);

		if (min_val < -1)
			sprintf(buf, "%Ld", (LONG LONG)n->min_val);
		else
			sprintf(buf, "%Lu", (unsigned LONG_LONG)n->min_val);
		len = strlen(buf);
		if (len > result_max_len)
			result_max_len = len;
	#else
		if (min_val < -1)
			sprintf(buf, "%d", (int)n->max_val);
		else
			sprintf(buf, "%u", (unsigned int)n->max_val);
		result_max_len = strlen(buf);

		if (min_val < -1)
			sprintf(buf, "%d", (int)n->min_val);
		else
			sprintf(buf, "%u", (unsigned int)n->min_val);
		len = strlen(buf);
		if (len > result_max_len)
			result_max_len = len;
	#endif

	/* Set up result object. */
	d->proc = d_calc_edit_result_proc;
	d->x = d[-1].x + d[-1].w + 6;
	d->y = d[-1].y + 3;
	d->w = widest_digit * result_max_len;
	d->fg = _dime_text_color;
	d->bg = _dime_background_color;
	d->dp = DIME_MALLOC(result_max_len * uwidth_max(U_CURRENT) + ucwidth(0),
							  char);
	d->dp2 = _dime_font;

	return 0;
}



/* dialogf_create_int:
*/
int dialogf_create_int(DIALOG *d, char *desc, char *arg_text,
							  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_integer(d, arg_text, args, INT_MIN, INT_MAX);
}



/* dialogf_store_int:
*/
void dialogf_store_int(DIALOG *d)
{
	*(int *)d->dp3 = dialogf_clamp_calc(d, INT_MIN, INT_MAX, TRUE);
}



/* dialogf_create_uint:
*/
int dialogf_create_uint(DIALOG *d, char *desc, char *arg_text,
								dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_integer(d, arg_text, args, 0, UINT_MAX);
}



/* dialogf_store_uint:
*/
void dialogf_store_uint(DIALOG *d)
{
	*(unsigned int *)d->dp3 = dialogf_clamp_calc(d, 0, UINT_MAX, TRUE);
}



/* dialogf_create_short:
*/
int dialogf_create_short(DIALOG *d, char *desc, char *arg_text,
								 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_integer(d, arg_text, args, SHRT_MIN, SHRT_MAX);
}



/* dialogf_store_short:
*/
void dialogf_store_short(DIALOG *d)
{
	*(short *)d->dp3 = dialogf_clamp_calc(d, SHRT_MIN, SHRT_MAX, TRUE);
}



/* dialogf_create_ushort:
*/
int dialogf_create_ushort(DIALOG *d, char *desc, char *arg_text,
								  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_integer(d, arg_text, args, 0, USHRT_MAX);
}



/* dialogf_store_ushort:
*/
void dialogf_store_ushort(DIALOG *d)
{
	*(unsigned short *)d->dp3 = dialogf_clamp_calc(d, 0, USHRT_MAX, TRUE);
}



/* dialogf_create_char:
*/
int dialogf_create_char(DIALOG *d, char *desc, char *arg_text,
								dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_integer(d, arg_text, args, CHAR_MIN, CHAR_MAX);
}



/* dialogf_store_char:
*/
void dialogf_store_char(DIALOG *d)
{
	*(char *)d->dp3 = dialogf_clamp_calc(d, CHAR_MIN, CHAR_MAX, TRUE);
}



/* dialogf_create_uchar:
*/
int dialogf_create_uchar(DIALOG *d, char *desc, char *arg_text,
								 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_integer(d, arg_text, args, 0, UCHAR_MAX);
}



/* dialogf_store_uchar:
*/
void dialogf_store_uchar(DIALOG *d)
{
	*(unsigned char *)d->dp3 = dialogf_clamp_calc(d, 0, UCHAR_MAX, TRUE);
}



/* long long is a bit tricky to implement and doesn't work yet. */
#ifdef DIME_LONG_LONG
/* dialogf_create_longlong:
*/
int dialogf_create_longlong(DIALOG *d, char *desc, char *arg_text,
									 dime_va_list_p args)
{
	return dialogf_create_integer(d, arg_text, args,
											LONG_LONG_MIN, LONG_LONG_MAX);
}



/* dialogf_store_longlong:
*/
void dialogf_store_longlong(DIALOG *d)
{
	*(LONG_LONG *)d->dp3 = dialogf_clamp_calc(d, LONG_LONG_MIN, LONG_LONG_MAX,
															TRUE);
}



/* dialogf_create_ulonglong:
*/
int dialogf_create_ulonglong(DIALOG *d, char *desc, char *arg_text,
									  dime_va_list_p args)
{
	return dialogf_create_integer(d, arg_text, args, 0, ULONG_LONG_MAX);
}



/* dialogf_store_ulonglong:
*/
void dialogf_store_ulonglong(DIALOG *d)
{
	*(unsigned LONG_LONG *)d->dp3 = dialogf_clamp_calc(d, 0, ULONG_LONG_MAX,
																		TRUE);
}
#endif