/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dline.c:
	Dialogf functions for the %line format, which does nothing but displaying
	a horizontal delimiter line.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	dialogf(..., "%line[]");
*/
#include <stdarg.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_line:
*/
int dialogf_create_line(DIALOG *d, char *desc, char *arg_text,
								dime_va_list_p args)
{
	DIME_UNUSED_ARG(args);
	DIME_ASSERT(ugetc(arg_text) == 0,
					("No arguments must be given to %%line"));

	if (_dialogf_pass_index == 0) {
		d->w = 0;
		d[1].w = 0;
		return 3;
	}
	else if (_dialogf_pass_index == 1) {
		/* Create description when we know where left edge of dialog is. */
		char *tmp = DIME_MALLOC(ustrsizez(desc), char);

		dialogf_create_desc(d, d->y, ustrcpy(tmp, desc));//ustrdup(desc));
		d->x = _dime_dialog->x + dime_window_border_thickness;
		d->d1 = ALIGN_LEFT;
		return 2;
	}
	else if (!_dialogf_final_pass)
		return 2;
	else {
		/* We need a final pass to create the line since the creation of
			a description for another line may have widened the dialog even
			more. */

		/* Create line. */
		d++;
		d->proc = d_line_proc;
		d->x = d[-1].x;
		if (d[-1].w)
			d->x += d[-1].w + 3;
		d->y = d[-1].y + d[-1].h / 2;
		if (d[-1].h > 2)
			d->y--;
		d->w = _dime_dialog->x + _dime_dialog->w -
				 dime_window_border_thickness - d->x;
		d->h = 2;
		return 0;
	}
}