/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dpfloat.c:
	Functions for all plain floating point formats, ie %pfloat and %pdouble.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	float my_float;
	dialogf(..., "Select a number:%pfloat[]", &my_float);

	double my_double;
	dialogf(..., "Select a number:%pdouble[]", &my_double);
*/
#include <math.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_pfloatingpoint:
*/
int dialogf_create_pfloatingpoint(DIALOG *d, char *arg_text,
											 dime_va_list_p args, int is_double)
{
	DIALOGF_EDIT_NUMBER *n;
	int state;
	double val;

	n = DIME_MALLOC(1, DIALOGF_EDIT_NUMBER);

	/* Parse arguments. */
	state = dime_get_double_arg(&arg_text, &val);
	DIME_ASSERT(state & DIME_ARG_COMMA,
					("',' expected in modifier: [%s]", arg_text));
	if (state & DIME_ARG_READ)
		n->min_val = val;
	else
		n->min_val = -HUGE_VAL;

	state = dime_get_double_arg(&arg_text, &val);
	DIME_ASSERT(state & DIME_ARG_END,
					("Garbage at end of modifier ('[' expected): [%s]", arg_text));
	if (state & DIME_ARG_READ)
		n->max_val = val;
	else
		n->max_val = HUGE_VAL;

	n->result_object = d + 1;

	/* Set up object. */
	d->proc = d_dime_edit_proc;
	d->h += 6;
	d->fg = _dime_text_color;
	d->bg = _dime_edit_field_color;
	d->d1 = 256;
	d->dp = DIME_MALLOC(256 * uwidth_max(U_CURRENT) + ucwidth(0), char);
	d->dp2 = n;

	if (is_double) {
		d->dp3 = dime_va_arg_p(args, double *);
		n->is_float = FALSE;
	}
	else {
		d->dp3 = dime_va_arg_p(args, float *);
		n->is_float = TRUE;
	}

	return 0;
}



/* dialogf_create_pdouble:
*/
int dialogf_create_pdouble(DIALOG *d, char *desc, char *arg_text,
									dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pfloatingpoint(d, arg_text, args, TRUE);
}



/* dialogf_store_pdouble:
*/
void dialogf_store_pdouble(DIALOG *d)
{
	*(double *)d->dp3 = dialogf_clamp_plain(d, -HUGE_VAL, HUGE_VAL, FALSE);
}



/* dialogf_reset_double:
*/
void dialogf_reset_double(DIALOG *d)
{
	udouble2string(*(double *)d->dp3, (char *)d->dp);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_pfloat:
*/
int dialogf_create_pfloat(DIALOG *d, char *desc, char *arg_text,
								  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pfloatingpoint(d, arg_text, args, FALSE);
}



/* dialogf_store_pfloat:
*/
void dialogf_store_pfloat(DIALOG *d)
{
	*(float *)d->dp3 = dialogf_clamp_plain(d, -HUGE_VAL, HUGE_VAL, FALSE);
}



/* dialogf_reset_float:
*/
void dialogf_reset_float(DIALOG *d)
{
	udouble2string(*(float *)d->dp3, (char *)d->dp);
	d->d2 = ustrlen((char *)d->dp);
}