/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	dpintege.c:
	Functions common to all plain integer types.

	By Sven Sandberg

	See readme.txt for more information about Dime.


	int my_int = 0;
	dialogf(..., "Select a number between -4711 and 471100:%pint[-4711,471100]",
			  &my_int);

	unsigned int my_uint = 0;
	dialogf(..., "Select a positive number:%uint[,]",
			  &my_uint);

	short my_short = 0;
	dialogf(..., "Select a number between -4711 and 4711:%pshort[-4711,4711]",
			  &my_short);

	unsigned short my_ushort = 0;
	dialogf(..., "Select a number between 0 and 4711:%pushort[,4711]",
			  &my_ushort);

	char my_char = 0;
	dialogf(..., "Select a number between -47 and 11:%pchar[-47,11]",
			  &my_char);

	unsigned char my_uchar = 0;
	dialogf(..., "Select a number between 11 and 47:%puchar[11,47]",
			  &my_uchar);
*/
#include <limits.h>
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dialogf_create_pinteger:
*/
int dialogf_create_pinteger(DIALOG *d, char *arg_text, dime_va_list_p args,
									 double min_val, double max_val)
{
	DIALOGF_EDIT_NUMBER *n;
	int state;
	int val;

	n = DIME_MALLOC(1, DIALOGF_EDIT_NUMBER);
	n->is_signed = min_val < -1 ? TRUE : FALSE;

	/* Parse arguments. */
	state = dime_get_int_arg(&arg_text, &val);
	DIME_ASSERT(state & DIME_ARG_COMMA,
					("',' expected in modifier: [%s]", arg_text));
	if (state & DIME_ARG_READ)
		n->min_val = val;
	else
		n->min_val = min_val;

	state = dime_get_int_arg(&arg_text, &val);
	DIME_ASSERT(state & DIME_ARG_END,
					("Garbage at end of modifier to %%int ('[' expected): [%s]",
					 arg_text));
	if (state & DIME_ARG_READ)
		n->max_val = val;
	else
		n->max_val = max_val;

	n->result_object = d + 1;

	/* Set up object. */
	d->proc = d_dime_edit_proc;
	d->h += 6;
	d->fg = _dime_text_color;
	d->bg = _dime_edit_field_color;
	d->d1 = 256;
	d->dp = DIME_MALLOC(256 * uwidth_max(U_CURRENT) + ucwidth(0), char);
	d->dp2 = n;
/* long long is a bit tricky to implement and doesn't work yet. */
#ifdef DIME_LONG_LONG
	if (max_val >= LONG_LONG_MAX)
		d->dp3 = dime_va_arg_p(args, LONG_LONG *);
	else
#endif
	if (max_val >= INT_MAX)
		d->dp3 = dime_va_arg_p(args, int *);
	else if (max_val >= SHRT_MAX)
		d->dp3 = dime_va_arg_p(args, short *);
	else
		d->dp3 = dime_va_arg_p(args, char *);

	d->d2 = ustrlen((char *)d->dp);

	return 0;
}



/* dialogf_create_pint:
*/
int dialogf_create_pint(DIALOG *d, char *desc, char *arg_text,
								dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pinteger(d, arg_text, args, INT_MIN, INT_MAX);
}



/* dialogf_store_pint:
*/
void dialogf_store_pint(DIALOG *d)
{
	*(int *)d->dp3 = dialogf_clamp_plain(d, INT_MIN, INT_MAX, TRUE);
}



/* dialogf_reset_int:
*/
void dialogf_reset_int(DIALOG *d)
{
	usprintf((char *)d->dp, "%d", *(int *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_puint:
*/
int dialogf_create_puint(DIALOG *d, char *desc, char *arg_text,
								 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pinteger(d, arg_text, args, 0, UINT_MAX);
}



/* dialogf_store_puint:
*/
void dialogf_store_puint(DIALOG *d)
{
	*(unsigned int *)d->dp3 = dialogf_clamp_plain(d, 0, UINT_MAX, TRUE);
}



/* dialogf_reset_uint:
*/
void dialogf_reset_uint(DIALOG *d)
{
	usprintf((char *)d->dp, "%u", *(unsigned int *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_pshort:
*/
int dialogf_create_pshort(DIALOG *d, char *desc, char *arg_text,
								  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pinteger(d, arg_text, args, SHRT_MIN, SHRT_MAX);
}



/* dialogf_store_pshort:
*/
void dialogf_store_pshort(DIALOG *d)
{
	*(short *)d->dp3 = dialogf_clamp_plain(d, SHRT_MIN, SHRT_MAX, TRUE);
}



/* dialogf_reset_short:
*/
void dialogf_reset_short(DIALOG *d)
{
	usprintf((char *)d->dp, "%d", (int)*(short *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_pushort:
*/
int dialogf_create_pushort(DIALOG *d, char *desc, char *arg_text,
									dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pinteger(d, arg_text, args, 0, USHRT_MAX);
}



/* dialogf_store_pushort:
*/
void dialogf_store_pushort(DIALOG *d)
{
	*(unsigned short *)d->dp3 = dialogf_clamp_plain(d, 0, USHRT_MAX, TRUE);
}



/* dialogf_reset_ushort:
*/
void dialogf_reset_ushort(DIALOG *d)
{
	usprintf((char *)d->dp, "%u", (int)*(unsigned short *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_pchar:
*/
int dialogf_create_pchar(DIALOG *d, char *desc, char *arg_text,
								 dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pinteger(d, arg_text, args, CHAR_MIN, CHAR_MAX);
}



/* dialogf_store_pchar:
*/
void dialogf_store_pchar(DIALOG *d)
{
	*(char *)d->dp3 = dialogf_clamp_plain(d, CHAR_MIN, CHAR_MAX, TRUE);
}



/* dialogf_reset_char:
*/
void dialogf_reset_char(DIALOG *d)
{
	usprintf((char *)d->dp, "%d", (int)*(char *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_puchar:
*/
int dialogf_create_puchar(DIALOG *d, char *desc, char *arg_text,
								  dime_va_list_p args)
{
	DIME_UNUSED_ARG(desc);
	return dialogf_create_pinteger(d, arg_text, args, 0, UCHAR_MAX);
}



/* dialogf_store_puchar:
*/
void dialogf_store_puchar(DIALOG *d)
{
	*(unsigned char *)d->dp3 = dialogf_clamp_plain(d, 0, UCHAR_MAX, TRUE);
}



/* dialogf_reset_uchar:
*/
void dialogf_reset_uchar(DIALOG *d)
{
	usprintf((char *)d->dp, "%u", (int)*(unsigned char *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* long long is a bit tricky to implement and doesn't work yet. */
#ifdef DIME_LONG_LONG
/* dialogf_create_plonglong:
*/
int dialogf_create_plonglong(DIALOG *d, char *desc, char *arg_text,
									  dime_va_list_p args)
{
	return dialogf_create_pinteger(d, arg_text, args, LONG_LONG_MIN,
																	  LONG_LONG_MAX);
}



/* dialogf_store_plonglong:
*/
void dialogf_store_plonglong(DIALOG *d)
{
	*(LONG_LONG *)d->dp3 = dialogf_clamp_plain(d, LONG_LONG_MIN,
															 LONG_LONG_MAX, TRUE);
}



/* dialogf_reset_long_long:
*/
void dialogf_reset_long_long(DIALOG *d)
{
	usprintf((char *)d->dp, "%Ld", *(LONG_LONG *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}



/* dialogf_create_pulonglong:
*/
int dialogf_create_pulonglong(DIALOG *d, char *desc, char *arg_text,
										dime_va_list_p args)
{
	return dialogf_create_pinteger(d, arg_text, args, 0, ULONG_LONG_MAX);
}



/* dialogf_store_pulonglong:
*/
void dialogf_store_pulonglong(DIALOG *d)
{
	*(unsigned LONG_LONG *)d->dp3 = dialogf_clamp_plain(d, 0, ULONG_LONG_MAX,
																		 TRUE);
}



/* dialogf_reset_ulong_long:
*/
void dialogf_reset_ulong_long(DIALOG *d)
{
	usprintf((char *)d->dp, "%Lu", *(LONG_LONG *)d->dp3);
	d->d2 = ustrlen((char *)d->dp);
}
#endif