/*  ___  _
	|   \(_)_ __  ___
	| |) | | '  \/ -_)
	|___/|_|_|_|_\___|

	parsemod.c:
	Functions for parsing modifiers to format strings.

	By Sven Sandberg

	See readme.txt for more information about Dime.
*/
#include <allegro.h>

#ifdef FORTIFY
#include "fortify2.h"
#endif

#include "dime.h"
#include "dime/dimeint.h"



/* dime_get_string_arg:
	Helper to extract a string from an argument.
*/
int dime_get_string_arg(char **arg, char *out, int max_size)
{
	char *t;
	int c;

	max_size -= uwidth_max(U_CURRENT) - ucwidth(0);

	t = out;
	for (c = ugetx(arg); c && (c != ','); c = ugetx(arg)) {
		if (c == '%') {
			c = ugetx(arg);
			DIME_ASSERT(c, ("Format string ended unexpectedly inside an "
								 "escape sequence ('%%') in a semicolon-separated "
								 "modifier."));
		}
		DIME_ASSERT(t - out < max_size,
						("String modifier to format in dialogf() too long. "
						 "Contact the author so he can increase the buffer "
						 "size in next version of Dime! (See readme.txt for "
						 "contact info.)"));
		t += usetc(t, c);
	}
	usetc(t, 0);

	return ((t - out > 0) ? DIME_ARG_READ : 0) |
			 ((c == ',') ? DIME_ARG_COMMA : DIME_ARG_END);
}



/* get_strlist_arg_worker:
*/
static int get_strlist_arg_worker(int just_count, char **arg_text,
											 int *out_num, char ***out_list)
{

	char buf[4096];
	int c;
	char *p;
	int ret = 0;
	char *my_arg_text;
	int string_i = 0;

	my_arg_text = *arg_text;
	p = buf;

	/* Loop through characters. */
	for (c = ugetx(&my_arg_text); ; c = ugetx(&my_arg_text)) {
		/* Escape sequence beginning with '%' */
		if (c == '%') {
			c = ugetx(&my_arg_text);
			DIME_ASSERT(c, ("Format string ended unexpectedly inside an "
								 "escape sequence ('%%') in a semicolon-separated "
								 "modifier."));
		}
		/* End of substring. */
		else if ((c == ';') || (c == ',') || (c == 0)) {
			if (ret || (c == ';')) {
				if (just_count)
					(*out_num)++;
				else {
					usetc(p, 0);
					(*out_list)[string_i++] = ustrdup(buf);
				}
			}
			if (c == 0)
				return ret | DIME_ARG_END;
			else if (c == ',')
				return ret | DIME_ARG_COMMA;
			p = buf;
			continue;
		}
		ret = DIME_ARG_READ;
		DIME_ASSERT((p - buf) + ucwidth(c) + ucwidth(0) < (int)sizeof(buf),
						("Semicolon-separated list item was too long. Please "
						 "contact the maintainer of Dime so that he can increase "
						 "the buffer size in next version of Dime! (See readme.txt"
						 "for contact info.)"));
		p += usetc(p, c);
	}
}



/* dime_get_strlist_arg:
	Parses a semicolon-separated list of strings.
*/
int dime_get_strlist_arg(char **arg_text, int *out_num, char ***out_list)
{
	*out_num = 0;
	get_strlist_arg_worker(TRUE, arg_text, out_num, out_list);
	*out_list = DIME_MALLOC(*out_num, char *);
	return get_strlist_arg_worker(FALSE, arg_text, out_num, out_list);
}



/* dime_get_int_arg:
	Helper to extract an integer from an argument.
*/
int dime_get_int_arg(char **arg, int *out)
{
	int val;
	int ret;
	char *endp;

	val = ustrtol(*arg, &endp, 0);

	if (endp == *arg) {
		ret = 0;
		*out = 0;
	}
	else {
		ret = DIME_ARG_READ;
		*out = val;
	}

	*arg = endp;

	if (ugetc(endp) == 0)
		return ret | DIME_ARG_END;
	else if (ugetc(endp) == ',') {
		*arg += ucwidth(',');
		return ret | DIME_ARG_COMMA;
	}
	else if (uisdigit(ugetc(endp)))
		return ret | DIME_ARG_DIGIT;
	else
		return ret | DIME_ARG_NONDIGIT;
}



/* dime_get_double_arg:
	Helper to extract a double from an argument.
*/
int dime_get_double_arg(char **arg, double *out)
{
	double val;
	int ret;
	char *endp;

	val = ustrtod(*arg, &endp);

	if (endp == *arg) {
		ret = 0;
		*out = 0;
	}
	else {
		ret = DIME_ARG_READ;
		*out = val;
	}

	*arg = endp;

	if (ugetc(endp) == 0)
		return ret | DIME_ARG_END;
	else if (ugetc(endp) == ',') {
		*arg += ucwidth(',');
		return ret | DIME_ARG_COMMA;
	}
	else if (uisdigit(ugetc(endp)))
		return ret | DIME_ARG_DIGIT;
	else
		return ret | DIME_ARG_NONDIGIT;
}