# zipup.sh
#
# Script to generate the distribution zips from dos.
# This is not exactly the most portable or beautiful way of doing it, but it
# works on my machine.
#
# Requires zip, rm, sed, tr, whisped (my own tool for converting tabs,
# newline characters and the like) and of course sh to execute the script.

if [ -d dime.old -o -d dime.new ]; then
	echo remove dime.new and dime.old first
	exit 1
fi

echo clean up...
./fixdjgpp.bat --quick
make clean
make rewhisp

# Don't generate dependencies if a parameter was given.
# This is because I often realize that I missed something essential _after_
# calling zipup.sh, so this saves lots of time if the dependencies don't need to
# be rebuilt
if [ "$1" = "--no-depend" -o "$2" = "--no-depend" ]; then
	./fixdjgpp.bat
else
	echo generate dependencies...
	./fixdll.bat --quick
	./fixming.bat --quick
	make depend
	./fixmsvc.bat --quick
	make depend
	./fixwat.bat --quick
	make depend
	./fixunix.sh --quick
	make depend
	./fixdjgpp.bat
	make depend
fi

echo generate other files for the distribution...
make readme.txt docs/readme.info docs/readme.html docs/readme.rtf "LFN=y"

echo whispedify everything...
make rewhisp


echo remove old files...
rm -fv dime.zip

echo copy default makefile...
cp emptymak makefile

echo create zip...
if [ "$1" = "--pkzip" -o "$2" = "--pkzip" ]; then
	rm dimetmp.zip
	
	for file in */tmpfile.txt */*/tmpfile.txt */*/*/tmpfile.txt */*/*/*/tmpfile.txt \
					fix* makefile* obj/*/*/*.dep lib/*/*.def zipup.sh \
			  		readme.txt docs/*._tx docs/readme.html docs/readme.info docs/readme.rtf \
			  		src/*.c examples/ex*.c \
			  		include/dime.h include/dime/*.h; do
		pkzip -a -ex -P dimetmp.zip $file
	done
	
	# Unzip the temporary zip.
	echo unzip temporary zip...
	mkdir zipup.tmp
	mkdir zipup.tmp/dime
	cd zipup.tmp/dime
	pkunzip -d ../../dimetmp.zip
	rm -fv ../../dimetmp.zip
	
	# Tolowerize files.
	echo tolowerize files...
	tolower -q -p +*
	
	# Copy the default makefile to here.
	echo copy temporary makefile to dimes directory...
	cp ../../emptymak makefile
	cd ..
	
	# Create an archive with lowercase directories, appropriate makefile and
	# everything in the dime/ directory.
	echo finally, build the distribution zip!
	pkzip -a -ex -P -r ../dime.zip *.*
	cd ..
	rm -fr zipup.tmp
else
	cd ..

	zip -9 dime/dime.zip \
			 dime/*/tmpfile.txt dime/*/*/tmpfile.txt \
			 dime/*/*/*/tmpfile.txt dime/*/*/*/*/tmpfile.txt \
			 dime/fix* dime/makefile* dime/emptymak \
			 dime/obj/*/*/*.dep dime/lib/*/*.def dime/zipup.sh \
			 dime/readme.txt dime/docs/*._tx dime/docs/readme.html \
			 dime/docs/readme.info dime/docs/readme.rtf \
			 dime/src/*.c dime/examples/ex*.c \
		  	 dime/include/dime.h dime/include/dime/*.h

	cd dime
fi

echo generate html files for the homepage...
make docs

echo all done!
